from datetime import date, datetime
from typing import Any, Dict, List, Optional, Union, overload

from github.AuthenticatedUser import AuthenticatedUser
from github.Branch import Branch
from github.Clones import Clones
from github.Commit import Commit
from github.CommitComment import CommitComment
from github.Comparison import Comparison
from github.ContentFile import ContentFile
from github.Download import Download
from github.Event import Event
from github.GitBlob import GitBlob
from github.GitCommit import GitCommit
from github.GithubObject import CompletableGithubObject, _NotSetType
from github.GitRef import GitRef
from github.GitRelease import GitRelease
from github.GitReleaseAsset import GitReleaseAsset
from github.GitTag import GitTag
from github.GitTree import GitTree
from github.Hook import Hook
from github.InputGitAuthor import InputGitAuthor
from github.InputGitTreeElement import InputGitTreeElement
from github.Invitation import Invitation
from github.Issue import Issue
from github.IssueComment import IssueComment
from github.IssueEvent import IssueEvent
from github.Label import Label
from github.Milestone import Milestone
from github.NamedUser import NamedUser
from github.Notification import Notification
from github.Organization import Organization
from github.PaginatedList import PaginatedList
from github.Path import Path
from github.Permissions import Permissions
from github.Project import Project
from github.PullRequest import PullRequest
from github.PullRequestComment import PullRequestComment
from github.Referrer import Referrer
from github.RepositoryKey import RepositoryKey
from github.SourceImport import SourceImport
from github.Stargazer import Stargazer
from github.StatsCodeFrequency import StatsCodeFrequency
from github.StatsCommitActivity import StatsCommitActivity
from github.StatsContributor import StatsContributor
from github.StatsParticipation import StatsParticipation
from github.StatsPunchCard import StatsPunchCard
from github.Tag import Tag
from github.Team import Team
from github.View import View
from github.Workflow import Workflow
from github.WorkflowRun import WorkflowRun

class Repository(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _hub(
        self, mode: str, event: str, callback: str, secret: Union[str, _NotSetType]
    ) -> None: ...
    @property
    def _identity(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _legacy_convert_issue(self, attributes: Dict[str, Any]) -> Dict[str, Any]: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def allow_merge_commit(self) -> bool: ...
    @property
    def allow_rebase_merge(self) -> bool: ...
    @property
    def allow_squash_merge(self) -> bool: ...
    @property
    def archived(self) -> bool: ...
    def add_to_collaborators(
        self,
        collaborator: Union[str, NamedUser],
        permission: Union[str, _NotSetType] = ...,
    ) -> None: ...
    @property
    def archive_url(self) -> str: ...
    @property
    def assignees_url(self) -> str: ...
    @property
    def blobs_url(self) -> str: ...
    @property
    def branches_url(self) -> str: ...
    @property
    def clone_url(self) -> str: ...
    @property
    def collaborators_url(self) -> str: ...
    @property
    def comments_url(self) -> str: ...
    @property
    def commits_url(self) -> str: ...
    def compare(self, base: str, head: str) -> Comparison: ...
    @property
    def compare_url(self) -> str: ...
    @property
    def contents_url(self) -> str: ...
    @property
    def contributors_url(self) -> str: ...
    def create_file(
        self,
        path: str,
        message: str,
        content: str,
        branch: Union[str, _NotSetType] = ...,
        committer: Union[InputGitAuthor, _NotSetType] = ...,
        author: Union[InputGitAuthor, _NotSetType] = ...,
    ) -> Dict[str, Union[ContentFile, Commit]]: ...
    def create_git_blob(self, content: str, encoding: str) -> GitBlob: ...
    def create_git_commit(
        self,
        message: str,
        tree: GitTree,
        parents: List[GitCommit],
        author: Union[InputGitAuthor, _NotSetType] = ...,
        committer: Union[InputGitAuthor, _NotSetType] = ...,
    ) -> GitCommit: ...
    def create_git_ref(self, ref: str, sha: str) -> GitRef: ...
    def create_git_release(
        self,
        tag: str,
        name: str,
        message: str,
        draft: bool = ...,
        prerelease: bool = ...,
        target_commitish: Union[str, _NotSetType] = ...,
    ) -> GitRelease: ...
    def create_git_tag(
        self,
        tag: str,
        message: str,
        object: str,
        type: str,
        tagger: Union[InputGitAuthor, _NotSetType] = ...,
    ) -> GitTag: ...
    def create_git_tag_and_release(
        self,
        tag: str,
        tag_message: str,
        release_name: str,
        release_message: str,
        object: str,
        type: str,
        tagger: Union[InputGitAuthor, _NotSetType] = ...,
        draft: bool = ...,
        prerelease: bool = ...,
    ) -> GitRelease: ...
    def create_git_tree(
        self,
        tree: List[InputGitTreeElement],
        base_tree: Union[GitTree, _NotSetType] = ...,
    ) -> GitTree: ...
    def create_hook(
        self,
        name: str,
        config: Dict[str, str],
        events: Union[_NotSetType, List[str]] = ...,
        active: Union[bool, _NotSetType] = ...,
    ) -> Hook: ...
    def create_issue(
        self,
        title: str,
        body: Union[str, _NotSetType] = ...,
        assignee: Union[NamedUser, str, _NotSetType] = ...,
        milestone: Union[Milestone, _NotSetType] = ...,
        labels: Union[List[Label], _NotSetType, List[str]] = ...,
        assignees: Union[_NotSetType, List[str], List[NamedUser]] = ...,
    ) -> Issue: ...
    def create_key(
        self, title: str, key: str, read_only: bool = ...
    ) -> RepositoryKey: ...
    def create_label(
        self, name: str, color: str, description: Union[str, _NotSetType] = ...
    ) -> Label: ...
    def create_milestone(
        self,
        title: str,
        state: Union[str, _NotSetType] = ...,
        description: Union[str, _NotSetType] = ...,
        due_on: Union[date, _NotSetType] = ...,
    ) -> Milestone: ...
    def create_project(self, name: str, body: str = ...) -> Project: ...
    @overload
    def create_pull(self, title: str, body: str, base: str, head: str, maintainer_can_modify: Union[bool, _NotSetType], draft: bool, issue: _NotSetType) -> PullRequest: ...
    @overload
    def create_pull(self, title: _NotSetType, body: _NotSetType, base: str, head: str, maintainer_can_modify: _NotSetType, draft: bool, issue: Issue) -> PullRequest: ...
    def create_repository_dispatch(
        self, event_type: str, client_payload: Dict[str, str]
    ) -> bool: ...
    def create_source_import(
        self,
        vcs: str,
        vcs_url: str,
        vcs_username: Union[str, _NotSetType] = ...,
        vcs_password: Union[str, _NotSetType] = ...,
    ) -> SourceImport: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def default_branch(self) -> str: ...
    def delete(self) -> None: ...
    def delete_file(
        self,
        path: str,
        message: str,
        sha: str,
        branch: Union[str, _NotSetType] = ...,
        committer: Union[InputGitAuthor, _NotSetType] = ...,
        author: Union[InputGitAuthor, _NotSetType] = ...,
    ) -> Dict[str, Union[Commit, _NotSetType]]: ...
    @property
    def deployments_url(self) -> str: ...
    @property
    def description(self) -> str: ...
    def disable_automated_security_fixes(self) -> bool: ...
    def disable_vulnerability_alert(self) -> bool: ...
    @property
    def downloads_url(self) -> str: ...
    def edit(
        self,
        name: Optional[str] = ...,
        description: Union[str, _NotSetType] = ...,
        homepage: Union[str, _NotSetType] = ...,
        private: Union[bool, _NotSetType] = ...,
        has_issues: Union[bool, _NotSetType] = ...,
        has_projects: Union[bool, _NotSetType] = ...,
        has_wiki: Union[bool, _NotSetType] = ...,
        has_downloads: Union[bool, _NotSetType] = ...,
        default_branch: Union[str, _NotSetType] = ...,
        allow_squash_merge: Union[bool, _NotSetType] = ...,
        allow_merge_commit: Union[bool, _NotSetType] = ...,
        allow_rebase_merge: Union[bool, _NotSetType] = ...,
        archived: Union[bool, _NotSetType] = ...,
    ) -> None: ...
    def enable_automated_security_fixes(self) -> bool: ...
    def enable_vulnerability_alert(self) -> bool: ...
    @property
    def events_url(self) -> str: ...
    @property
    def fork(self) -> bool: ...
    @property
    def forks(self) -> int: ...
    @property
    def forks_count(self) -> int: ...
    @property
    def forks_url(self) -> str: ...
    @property
    def full_name(self) -> str: ...
    def get_archive_link(
        self, archive_format: str, ref: Union[str, _NotSetType] = ...
    ) -> str: ...
    def get_assignees(self) -> PaginatedList[NamedUser]: ...
    def get_branch(self, branch: str) -> Branch: ...
    def get_branches(self) -> PaginatedList[Branch]: ...
    def get_clones_traffic(
        self, per: Union[str, _NotSetType] = ...
    ) -> Dict[str, Union[int, List[Clones]]]: ...
    def get_collaborator_permission(
        self, collaborator: Union[str, NamedUser]
    ) -> str: ...
    def get_collaborators(
        self, affiliation: Union[str, _NotSetType] = ...
    ) -> PaginatedList[NamedUser]: ...
    def get_comment(self, id: int) -> CommitComment: ...
    def get_comments(self) -> PaginatedList[CommitComment]: ...
    def get_commit(self, sha: str) -> Commit: ...
    def get_commits(
        self,
        sha: Union[str, _NotSetType] = ...,
        path: Union[str, _NotSetType] = ...,
        since: Union[_NotSetType, datetime] = ...,
        until: Union[_NotSetType, datetime] = ...,
        author: Union[AuthenticatedUser, NamedUser, str, _NotSetType] = ...,
    ) -> PaginatedList[Commit]: ...
    def get_contents(
        self, path: str, ref: Union[str, _NotSetType] = ...
    ) -> Union[List[ContentFile], ContentFile]: ...
    def get_contributors(
        self, anon: Union[str, _NotSetType] = ...
    ) -> PaginatedList[NamedUser]: ...
    def get_dir_contents(
        self, path: str, ref: Union[str, _NotSetType] = ...
    ) -> List[ContentFile]: ...
    def get_download(self, id: int) -> Download: ...
    def get_downloads(self) -> PaginatedList[Download]: ...
    def get_events(self) -> PaginatedList[Event]: ...
    def get_forks(self) -> PaginatedList[Repository]: ...
    def get_git_blob(self, sha: str) -> GitBlob: ...
    def get_git_commit(self, sha: str) -> GitCommit: ...
    def get_git_ref(self, ref: str) -> GitRef: ...
    def get_git_refs(self) -> PaginatedList[GitRef]: ...
    def get_git_tag(self, sha: str) -> GitTag: ...
    def get_git_tree(
        self, sha: str, recursive: Union[bool, _NotSetType] = ...
    ) -> GitTree: ...
    def get_hook(self, id: int) -> Hook: ...
    def get_hooks(self) -> PaginatedList[Hook]: ...
    def get_issue(self, number: int) -> Issue: ...
    def get_issues(
        self,
        milestone: Union[Milestone, str, _NotSetType] = ...,
        state: Union[str, _NotSetType] = ...,
        assignee: Union[NamedUser, str, _NotSetType] = ...,
        mentioned: Union[_NotSetType, NamedUser] = ...,
        labels: Union[List[str], List[Label], _NotSetType] = ...,
        sort: Union[str, _NotSetType] = ...,
        direction: Union[str, _NotSetType] = ...,
        since: Union[_NotSetType, datetime] = ...,
        creator: Union[NamedUser, _NotSetType] = ...,
    ) -> PaginatedList[Issue]: ...
    def get_issues_comments(
        self,
        sort: Union[str, _NotSetType] = ...,
        direction: Union[str, _NotSetType] = ...,
        since: Union[_NotSetType, datetime] = ...,
    ) -> PaginatedList[IssueComment]: ...
    def get_issues_event(self, id: int) -> IssueEvent: ...
    def get_issues_events(self) -> PaginatedList[IssueEvent]: ...
    def get_key(self, id: int) -> RepositoryKey: ...
    def get_keys(self) -> PaginatedList[RepositoryKey]: ...
    def get_label(self, name: str) -> Label: ...
    def get_labels(self) -> PaginatedList[Label]: ...
    def get_languages(self) -> Dict[str, int]: ...
    def get_latest_release(self) -> GitRelease: ...
    def get_license(self) -> ContentFile: ...
    def get_milestone(self, number: int) -> Milestone: ...
    def get_milestones(
        self,
        state: Union[str, _NotSetType] = ...,
        sort: Union[str, _NotSetType] = ...,
        direction: Union[str, _NotSetType] = ...,
    ) -> PaginatedList[Milestone]: ...
    def get_network_events(self) -> PaginatedList[Event]: ...
    def get_notifications(
        self,
        all: Union[bool, _NotSetType] = ...,
        participating: Union[bool, _NotSetType] = ...,
        since: Union[datetime, _NotSetType] = ...,
        before: Union[datetime, _NotSetType] = ...,
    ) -> PaginatedList[Notification]: ...
    def get_pending_invitations(self) -> PaginatedList[Invitation]: ...
    def get_projects(
        self, state: Union[str, _NotSetType] = ...
    ) -> PaginatedList[Project]: ...
    def get_pull(self, number: int) -> PullRequest: ...
    def get_pulls(
        self,
        state: Union[str, _NotSetType] = ...,
        sort: Union[str, _NotSetType] = ...,
        direction: Union[str, _NotSetType] = ...,
        base: Union[str, _NotSetType] = ...,
        head: Union[str, _NotSetType] = ...,
    ) -> PaginatedList[PullRequest]: ...
    def get_pulls_comments(
        self,
        sort: Union[str, _NotSetType] = ...,
        direction: Union[str, _NotSetType] = ...,
        since: Union[_NotSetType, datetime] = ...,
    ) -> PaginatedList[PullRequestComment]: ...
    def get_pulls_review_comments(
        self,
        sort: Union[str, _NotSetType] = ...,
        direction: Union[str, _NotSetType] = ...,
        since: Union[_NotSetType, datetime] = ...,
    ) -> PaginatedList[PullRequestComment]: ...
    def get_readme(self, ref: Union[str, _NotSetType] = ...) -> ContentFile: ...
    def get_release(self, id: Union[int, str]) -> GitRelease: ...
    def get_release_asset(self, id: int) -> GitReleaseAsset: ...
    def get_releases(self) -> PaginatedList[GitRelease]: ...
    def get_source_import(self) -> SourceImport: ...
    def get_stargazers(self) -> PaginatedList[NamedUser]: ...
    def get_stargazers_with_dates(self) -> PaginatedList[Stargazer]: ...
    def get_stats_code_frequency(self) -> Optional[List[StatsCodeFrequency]]: ...
    def get_stats_commit_activity(self) -> Optional[List[StatsCommitActivity]]: ...
    def get_stats_contributors(self) -> Optional[List[StatsContributor]]: ...
    def get_stats_participation(self) -> Optional[StatsParticipation]: ...
    def get_stats_punch_card(self) -> Optional[StatsPunchCard]: ...
    def get_subscribers(self) -> PaginatedList[NamedUser]: ...
    def get_tags(self) -> PaginatedList[Tag]: ...
    def get_teams(self) -> PaginatedList[Team]: ...
    def get_top_paths(self) -> List[Path]: ...
    def get_top_referrers(self) -> List[Referrer]: ...
    def get_topics(self) -> List[str]: ...
    def get_views_traffic(
        self, per: Union[str, _NotSetType] = ...
    ) -> Dict[str, Union[int, List[View]]]: ...
    def get_vulnerability_alert(self) -> bool: ...
    def get_watchers(self) -> PaginatedList[NamedUser]: ...
    def get_workflow(self, id_or_name: Union[str, int]) -> Workflow: ...
    def get_workflows(self) -> PaginatedList[Workflow]: ...
    def get_workflow_run(self, id_: int) -> WorkflowRun: ...
    def get_workflow_runs(self) -> PaginatedList[WorkflowRun]: ...
    @property
    def git_commits_url(self) -> str: ...
    @property
    def git_refs_url(self) -> str: ...
    @property
    def git_tags_url(self) -> str: ...
    @property
    def git_url(self) -> str: ...
    @property
    def has_downloads(self) -> bool: ...
    def has_in_assignees(self, assignee: Union[str, NamedUser]) -> bool: ...
    def has_in_collaborators(self, collaborator: Union[str, NamedUser]) -> bool: ...
    @property
    def has_issues(self) -> bool: ...
    @property
    def has_pages(self) -> bool: ...
    @property
    def has_projects(self) -> bool: ...
    @property
    def has_wiki(self) -> bool: ...
    @property
    def homepage(self) -> str: ...
    @property
    def hooks_url(self) -> str: ...
    @property
    def html_url(self) -> str: ...
    @property
    def id(self) -> int: ...
    @property
    def issue_comment_url(self) -> str: ...
    @property
    def issue_events_url(self) -> str: ...
    @property
    def issues_url(self) -> str: ...
    @property
    def keys_url(self) -> str: ...
    @property
    def labels_url(self) -> str: ...
    @property
    def language(self) -> str: ...
    @property
    def languages_url(self) -> str: ...
    def legacy_search_issues(self, state: str, keyword: str) -> List[Issue]: ...
    def mark_notifications_as_read(self, last_read_at: datetime = ...) -> None: ...
    @property
    def master_branch(self) -> Optional[str]: ...
    def merge(
        self, base: str, head: str, commit_message: Union[str, _NotSetType] = ...
    ) -> Optional[Commit]: ...
    @property
    def merges_url(self) -> str: ...
    @property
    def milestones_url(self) -> str: ...
    @property
    def mirror_url(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def network_count(self) -> int: ...
    @property
    def notifications_url(self) -> str: ...
    @property
    def open_issues(self) -> int: ...
    @property
    def open_issues_count(self) -> int: ...
    @property
    def organization(self) -> Organization: ...
    @property
    def owner(self) -> NamedUser: ...
    @property
    def parent(self) -> Repository: ...
    @property
    def permissions(self) -> Permissions: ...
    @property
    def private(self) -> bool: ...
    @property
    def pulls_url(self) -> str: ...
    @property
    def pushed_at(self) -> datetime: ...
    @property
    def releases_url(self) -> str: ...
    def remove_from_collaborators(
        self, collaborator: Union[str, NamedUser]
    ) -> None: ...
    def remove_invitation(self, invite_id: int) -> None: ...
    def replace_topics(self, topics: List[str]) -> None: ...
    @property
    def size(self) -> int: ...
    @property
    def source(self) -> Optional[Repository]: ...
    @property
    def ssh_url(self) -> str: ...
    @property
    def stargazers_count(self) -> int: ...
    @property
    def stargazers_url(self) -> str: ...
    @property
    def statuses_url(self) -> str: ...
    def subscribe_to_hub(
        self, event: str, callback: str, secret: Union[str, _NotSetType] = ...
    ) -> None: ...
    @property
    def subscribers_count(self) -> int: ...
    @property
    def subscribers_url(self) -> str: ...
    @property
    def subscription_url(self) -> str: ...
    @property
    def svn_url(self) -> str: ...
    @property
    def tags_url(self) -> str: ...
    @property
    def teams_url(self) -> str: ...
    @property
    def trees_url(self) -> str: ...
    def unsubscribe_from_hub(self, event: str, callback: str) -> None: ...
    def update_file(
        self,
        path: str,
        message: str,
        content: Union[bytes, str],
        sha: str,
        branch: Union[_NotSetType, str] = ...,
        committer: Union[_NotSetType, InputGitAuthor] = ...,
        author: Union[_NotSetType, InputGitAuthor] = ...,
    ) -> Dict[str, Union[ContentFile, Commit]]: ...
    @property
    def updated_at(self) -> datetime: ...
    @property
    def url(self) -> str: ...
    @property
    def watchers(self) -> int: ...
    @property
    def watchers_count(self) -> int: ...



from datetime import datetime
from typing import Any, Dict

from github.GithubObject import CompletableGithubObject
from github.NamedUser import NamedUser
from github.PaginatedList import PaginatedList
from github.ProjectColumn import ProjectColumn

class Project(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def body(self) -> str: ...
    @property
    def columns_url(self) -> str: ...
    def create_column(self, name: str) -> ProjectColumn: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def creator(self) -> NamedUser: ...
    def get_columns(self) -> PaginatedList[ProjectColumn]: ...
    @property
    def html_url(self) -> str: ...
    @property
    def id(self) -> int: ...
    @property
    def name(self) -> str: ...
    @property
    def node_id(self) -> str: ...
    @property
    def number(self) -> int: ...
    @property
    def owner_url(self) -> str: ...
    @property
    def state(self) -> str: ...
    @property
    def updated_at(self) -> datetime: ...
    @property
    def url(self) -> str: ...



from typing import Any, Dict, List

from github.GithubObject import CompletableGithubObject
from github.GitTreeElement import GitTreeElement

class GitTree(CompletableGithubObject):
    def __repr__(self) -> str: ...
    @property
    def _identity(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def sha(self) -> str: ...
    @property
    def tree(self) -> List[GitTreeElement]: ...
    @property
    def url(self) -> str: ...



# -*- coding: utf-8 -*-

############################ Copyrights and license ############################
#                                                                              #
# Copyright 2017 Nicolas Agustín Torres <nicolastrres@gmail.com>              #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import github.GithubObject
import github.NamedUser

from . import Consts


class Reaction(github.GithubObject.CompletableGithubObject):
    """
    This class represents Reactions. The reference can be found here https://developer.github.com/v3/reactions/
    """

    def __repr__(self):
        return self.get__repr__({"id": self._id.value, "user": self._user.value})

    @property
    def content(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._content)
        return self._content.value

    @property
    def created_at(self):
        """
        :type: datetime.datetime
        """
        self._completeIfNotSet(self._created_at)
        return self._created_at.value

    @property
    def id(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._id)
        return self._id.value

    @property
    def user(self):
        """
        :type: :class:`github.NamedUser.NamedUser`
        """
        self._completeIfNotSet(self._user)
        return self._user.value

    def delete(self):
        """
        :calls: `DELETE /reactions/:id <https://developer.github.com/v3/reactions/#delete-a-reaction>`_
        :rtype: None
        """
        self._requester.requestJsonAndCheck(
            "DELETE",
            self._parentUrl("") + "/reactions/" + str(self.id),
            headers={"Accept": Consts.mediaTypeReactionsPreview},
        )

    def _initAttributes(self):
        self._content = github.GithubObject.NotSet
        self._created_at = github.GithubObject.NotSet
        self._id = github.GithubObject.NotSet
        self._user = github.GithubObject.NotSet

    def _useAttributes(self, attributes):
        if "content" in attributes:  # pragma no branch
            self._content = self._makeStringAttribute(attributes["content"])
        if "created_at" in attributes:  # pragma no branch
            self._created_at = self._makeDatetimeAttribute(attributes["created_at"])
        if "id" in attributes:  # pragma no branch
            self._id = self._makeIntAttribute(attributes["id"])
        if "user" in attributes:  # pragma no branch
            self._user = self._makeClassAttribute(
                github.NamedUser.NamedUser, attributes["user"]
            )



from typing import Any, Dict, List, Union

from github.CommitCombinedStatus import CommitCombinedStatus
from github.CommitComment import CommitComment
from github.CommitStats import CommitStats
from github.CommitStatus import CommitStatus
from github.File import File
from github.GitCommit import GitCommit
from github.GithubObject import CompletableGithubObject, _NotSetType
from github.NamedUser import NamedUser
from github.PaginatedList import PaginatedList

class Commit(CompletableGithubObject):
    def __repr__(self) -> str: ...
    @property
    def _identity(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def author(self) -> NamedUser: ...
    @property
    def comments_url(self) -> str: ...
    @property
    def commit(self) -> GitCommit: ...
    @property
    def committer(self) -> NamedUser: ...
    def create_comment(
        self,
        body: str,
        line: Union[int, _NotSetType] = ...,
        path: Union[_NotSetType, str] = ...,
        position: Union[int, _NotSetType] = ...,
    ) -> CommitComment: ...
    def create_status(
        self,
        state: str,
        target_url: Union[_NotSetType, str] = ...,
        description: Union[_NotSetType, str] = ...,
        context: Union[_NotSetType, str] = ...,
    ) -> CommitStatus: ...
    @property
    def files(self) -> List[File]: ...
    def get_combined_status(self) -> CommitCombinedStatus: ...
    def get_comments(self) -> PaginatedList[GitCommit]: ...
    def get_statuses(self) -> PaginatedList[CommitStatus]: ...
    @property
    def html_url(self) -> str: ...
    @property
    def parents(self) -> List[Commit]: ...
    @property
    def sha(self) -> str: ...
    @property
    def stats(self) -> CommitStats: ...
    @property
    def url(self) -> str: ...



from datetime import datetime
from typing import Any, Dict

from github.GithubObject import CompletableGithubObject
from github.NamedUser import NamedUser
from github.PaginatedList import PaginatedList
from github.Reaction import Reaction

class IssueComment(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def body(self) -> str: ...
    def create_reaction(self, reaction_type: str) -> Reaction: ...
    @property
    def created_at(self) -> datetime: ...
    def delete(self) -> None: ...
    def edit(self, body: str) -> None: ...
    def get_reactions(self) -> PaginatedList[Reaction]: ...
    @property
    def html_url(self) -> str: ...
    @property
    def id(self) -> int: ...
    @property
    def issue_url(self) -> str: ...
    @property
    def updated_at(self) -> datetime: ...
    @property
    def url(self) -> str: ...
    @property
    def user(self) -> NamedUser: ...



from datetime import datetime
from typing import Any, Dict, List, Optional, Union

from github.Event import Event
from github.GithubObject import CompletableGithubObject, _NotSetType
from github.Hook import Hook
from github.Issue import Issue
from github.Label import Label
from github.Migration import Migration
from github.NamedUser import NamedUser
from github.PaginatedList import PaginatedList
from github.Plan import Plan
from github.Project import Project
from github.Repository import Repository
from github.Team import Team

class Organization(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    def add_to_members(
        self, member: NamedUser, role: Union[_NotSetType, str] = ...
    ) -> None: ...
    def add_to_public_members(self, public_member: NamedUser) -> None: ...
    @property
    def avatar_url(self) -> str: ...
    @property
    def billing_email(self) -> str: ...
    @property
    def blog(self) -> Optional[str]: ...
    @property
    def collaborators(self) -> int: ...
    @property
    def company(self) -> Optional[str]: ...
    def convert_to_outside_collaborator(self, member: NamedUser) -> None: ...
    def create_fork(self, repo: Repository) -> Repository: ...
    def create_hook(
        self,
        name: str,
        config: Dict[str, str],
        events: Union[_NotSetType, List[str]] = ...,
        active: Union[bool, _NotSetType] = ...,
    ) -> Hook: ...
    def create_migration(
        self,
        repos: List[str],
        lock_repositories: Union[bool, _NotSetType] = ...,
        exclude_attachments: Union[bool, _NotSetType] = ...,
    ) -> Migration: ...
    def create_repo(
        self,
        name: str,
        description: Union[str, _NotSetType] = ...,
        homepage: Union[str, _NotSetType] = ...,
        private: Union[bool, _NotSetType] = ...,
        has_issues: Union[bool, _NotSetType] = ...,
        has_wiki: Union[bool, _NotSetType] = ...,
        has_downloads: Union[bool, _NotSetType] = ...,
        has_projects: Union[bool, _NotSetType] = ...,
        team_id: Union[int, _NotSetType] = ...,
        auto_init: Union[bool, _NotSetType] = ...,
        license_template: Union[str, _NotSetType] = ...,
        gitignore_template: Union[str, _NotSetType] = ...,
        allow_squash_merge: Union[bool, _NotSetType] = ...,
        allow_merge_commit: Union[bool, _NotSetType] = ...,
        allow_rebase_merge: Union[bool, _NotSetType] = ...,
    ) -> Repository: ...
    def create_team(
        self,
        name: str,
        repo_names: Union[List[Repository], _NotSetType] = ...,
        permission: Union[str, _NotSetType] = ...,
        privacy: Union[str, _NotSetType] = ...,
        description: Union[str, _NotSetType] = ...,
    ) -> Team: ...
    @property
    def created_at(self) -> datetime: ...
    def delete_hook(self, id: int) -> None: ...
    @property
    def default_repository_permission(self) -> str: ...
    @property
    def description(self) -> str: ...
    @property
    def disk_usage(self) -> int: ...
    def edit(
        self,
        billing_email: Union[str, _NotSetType] = ...,
        blog: Union[str, _NotSetType] = ...,
        company: Union[str, _NotSetType] = ...,
        description: Union[str, _NotSetType] = ...,
        email: Union[str, _NotSetType] = ...,
        location: Union[str, _NotSetType] = ...,
        name: Union[str, _NotSetType] = ...,
    ) -> None: ...
    def edit_hook(
        self,
        id: int,
        name: str,
        config: Dict[str, str],
        events: Union[_NotSetType, List[str]] = ...,
        active: Union[bool, _NotSetType] = ...,
    ) -> Hook: ...
    @property
    def email(self) -> Optional[str]: ...
    @property
    def events_url(self) -> str: ...
    @property
    def followers(self) -> int: ...
    @property
    def following(self) -> int: ...
    def get_events(self) -> PaginatedList[Event]: ...
    def get_hooks(self) -> PaginatedList[Hook]: ...
    def get_issues(
        self,
        filter: Union[str, _NotSetType] = ...,
        state: Union[str, _NotSetType] = ...,
        labels: Union[List[Label], _NotSetType] = ...,
        sort: Union[str, _NotSetType] = ...,
        direction: Union[str, _NotSetType] = ...,
        since: Union[_NotSetType, datetime] = ...,
    ) -> PaginatedList[Issue]: ...
    def get_members(
        self,
        filter_: Union[str, _NotSetType] = ...,
        role: Union[str, _NotSetType] = ...,
    ) -> PaginatedList[NamedUser]: ...
    def get_migrations(self) -> PaginatedList[Migration]: ...
    def get_outside_collaborators(
        self, filter_: Union[str, _NotSetType] = ...
    ) -> PaginatedList[NamedUser]: ...
    def get_projects(
        self, state: Union[_NotSetType, str] = ...
    ) -> PaginatedList[Project]: ...
    def get_public_members(self) -> PaginatedList[NamedUser]: ...
    def get_repo(self, name: str) -> Repository: ...
    def get_repos(
        self,
        type: Union[str, _NotSetType] = ...,
        sort: Union[str, _NotSetType] = ...,
        direction: Union[str, _NotSetType] = ...,
    ) -> PaginatedList[Repository]: ...
    def get_team(self, id: int) -> Team: ...
    def get_team_by_slug(self, slug: str) -> Team: ...
    def get_teams(self) -> PaginatedList[Team]: ...
    @property
    def gravatar_id(self) -> str: ...
    def has_in_members(self, member: NamedUser) -> bool: ...
    def has_in_public_members(self, public_member: NamedUser) -> bool: ...
    @property
    def has_organization_projects(self) -> bool: ...
    @property
    def has_repository_projects(self) -> bool: ...
    @property
    def hooks_url(self) -> str: ...
    @property
    def html_url(self) -> str: ...
    @property
    def id(self) -> int: ...
    @property
    def issues_url(self) -> str: ...
    def invitations(self) -> PaginatedList[NamedUser]: ...
    def invite_user(
        self,
        user: Union[_NotSetType, NamedUser] = ...,
        email: Union[str, _NotSetType] = ...,
        role: Union[str, _NotSetType] = ...,
        teams: Union[List[Team], _NotSetType] = ...,
    ) -> None: ...
    @property
    def location(self) -> str: ...
    @property
    def login(self) -> str: ...
    @property
    def members_can_create_repositories(self) -> bool: ...
    @property
    def members_url(self) -> str: ...
    @property
    def name(self) -> Optional[str]: ...
    @property
    def owned_private_repos(self) -> int: ...
    @property
    def plan(self) -> Plan: ...
    @property
    def private_gists(self) -> int: ...
    @property
    def public_gists(self) -> int: ...
    @property
    def public_members_url(self) -> str: ...
    @property
    def public_repos(self) -> int: ...
    def remove_from_members(self, member: NamedUser) -> None: ...
    def remove_from_membership(self, member: NamedUser) -> None: ...
    def remove_from_public_members(self, public_member: NamedUser) -> None: ...
    def remove_outside_collaborator(self, collaborator: NamedUser) -> None: ...
    @property
    def repos_url(self) -> str: ...
    @property
    def total_private_repos(self) -> int: ...
    @property
    def two_factor_requirement_enabled(self) -> bool: ...
    @property
    def type(self) -> str: ...
    @property
    def updated_at(self) -> datetime: ...
    @property
    def url(self) -> str: ...



# -*- coding: utf-8 -*-

############################ Copyrights and license ############################
#                                                                              #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import github.GithubObject


class StatsCodeFrequency(github.GithubObject.NonCompletableGithubObject):
    """
    This class represents statistics of StatsCodeFrequencies. The reference can be found here http://developer.github.com/v3/repos/statistics/#get-the-number-of-additions-and-deletions-per-week
    """

    @property
    def week(self):
        """
        :type: datetime.datetime
        """
        return self._week.value

    @property
    def additions(self):
        """
        :type: int
        """
        return self._additions.value

    @property
    def deletions(self):
        """
        :type: int
        """
        return self._deletions.value

    def _initAttributes(self):
        self._week = github.GithubObject.NotSet
        self._additions = github.GithubObject.NotSet
        self._deletions = github.GithubObject.NotSet

    def _useAttributes(self, attributes):
        self._week = self._makeTimestampAttribute(attributes[0])
        self._additions = self._makeIntAttribute(attributes[1])
        self._deletions = self._makeIntAttribute(attributes[2])



from datetime import datetime
from typing import Any, Dict, NamedTuple, List

from github.GitCommit import GitCommit
from github.GithubObject import CompletableGithubObject
from github.PullRequest import PullRequest
from github.Repository import Repository

class TimingData(NamedTuple):
    billable: Dict[str, Dict[str, int]]
    run_duration_ms: int

class WorkflowRun(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def id(self) -> int: ...
    @property
    def head_branch(self) -> str: ...
    @property
    def head_sha(self) -> str: ...
    @property
    def run_number(self) -> int: ...
    @property
    def event(self) -> str: ...
    @property
    def status(self) -> str: ...
    @property
    def conclusion(self) -> str: ...
    @property
    def url(self) -> str: ...
    @property
    def html_url(self) -> str: ...
    @property
    def pull_requests(self) -> List[PullRequest]: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def updated_at(self) -> datetime: ...
    @property
    def jobs_url(self) -> str: ...
    @property
    def logs_url(self) -> str: ...
    @property
    def check_suite_url(self) -> str: ...
    @property
    def artifacts_url(self) -> str: ...
    @property
    def cancel_url(self) -> str: ...
    @property
    def rerun_url(self) -> str: ...
    @property
    def workflow_url(self) -> str: ...
    @property
    def head_commit(self) -> GitCommit: ...
    @property
    def repository(self) -> Repository: ...
    @property
    def head_repository(self) -> Repository: ...
    def cancel(self) -> bool: ...
    def rerun(self) -> bool: ...
    def timing(self) -> TimingData: ...



from typing import Any, Dict, List

from github.GitAuthor import GitAuthor
from github.GithubObject import CompletableGithubObject
from github.GitTree import GitTree

class GitCommit(CompletableGithubObject):
    def __repr__(self) -> str: ...
    @property
    def _identity(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def author(self) -> GitAuthor: ...
    @property
    def committer(self) -> GitAuthor: ...
    @property
    def html_url(self) -> str: ...
    @property
    def message(self) -> str: ...
    @property
    def parents(self) -> List[GitCommit]: ...
    @property
    def sha(self) -> str: ...
    @property
    def tree(self) -> GitTree: ...
    @property
    def url(self) -> str: ...



# -*- coding: utf-8 -*-

############################ Copyrights and license ############################
#                                                                              #
# Copyright 2017 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import github.GithubObject


class Invitation(github.GithubObject.CompletableGithubObject):
    """
    This class represents repository invitations. The reference can be found here https://developer.github.com/v3/repos/invitations/
    """

    def __repr__(self):
        return self.get__repr__({"id": self._id.value})

    @property
    def id(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._id)
        return self._id.value

    @property
    def permissions(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._permissions)
        return self._permissions.value

    @property
    def created_at(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._created_at)
        return self._created_at.value

    @property
    def invitee(self):
        """
        :type: NamedUser
        """
        self._completeIfNotSet(self._invitee)
        return self._invitee.value

    @property
    def inviter(self):
        """
        :type: NamedUser
        """
        self._completeIfNotSet(self._inviter)
        return self._inviter.value

    @property
    def url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._url)
        return self._url.value

    @property
    def html_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._html_url)
        return self._html_url.value

    @property
    def repository(self):
        """
        :type: Repository
        """
        self._completeIfNotSet(self._repository)
        return self._repository.value

    def _initAttributes(self):
        self._id = github.GithubObject.NotSet
        self._permissions = github.GithubObject.NotSet
        self._created_at = github.GithubObject.NotSet
        self._invitee = github.GithubObject.NotSet
        self._inviter = github.GithubObject.NotSet
        self._url = github.GithubObject.NotSet
        self._html_url = github.GithubObject.NotSet
        self._repository = github.GithubObject.NotSet

    def _useAttributes(self, attributes):
        if "repository" in attributes:  # pragma no branch
            self._repository = self._makeClassAttribute(
                github.Repository.Repository, attributes["repository"]
            )
        if "created_at" in attributes:  # pragma no branch
            self._created_at = self._makeDatetimeAttribute(attributes["created_at"])
        if "invitee" in attributes:  # pragma no branch
            self._invitee = self._makeClassAttribute(
                github.NamedUser.NamedUser, attributes["invitee"]
            )
        if "inviter" in attributes:  # pragma no branch
            self._inviter = self._makeClassAttribute(
                github.NamedUser.NamedUser, attributes["inviter"]
            )
        if "id" in attributes:  # pragma no branch
            self._id = self._makeIntAttribute(attributes["id"])

        if "permissions" in attributes:  # pragma no branch
            self._permissions = self._makeStringAttribute(attributes["permissions"])
        if "url" in attributes:  # pragma no branch
            self._url = self._makeStringAttribute(attributes["url"])
        if "html_url" in attributes:  # pragma no branch
            self._html_url = self._makeStringAttribute(attributes["html_url"])



# -*- coding: utf-8 -*-

############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Steve English <steve.english@navetas.com>                     #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 AKFish <akfish@gmail.com>                                     #
# Copyright 2013 Cameron White <cawhite@pdx.edu>                               #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2013 poulp <mathieu.nerv@gmail.com>                                #
# Copyright 2014 Tomas Radej <tradej@redhat.com>                               #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 E. Dunham <github@edunham.net>                                #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2017 Balázs Rostás <rostas.balazs@gmail.com>                       #
# Copyright 2017 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2017 Simon <spam@esemi.ru>                                         #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 bryanhuntesl <31992054+bryanhuntesl@users.noreply.github.com> #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
# Copyright 2018 itsbruce <it.is.bruce@gmail.com>                              #
# Copyright 2019 Rigas Papathanasopoulos <rigaspapas@gmail.com>                #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import datetime

import github.Authorization
import github.Event
import github.Gist
import github.GithubObject
import github.Invitation
import github.Issue
import github.Membership
import github.Migration
import github.NamedUser
import github.Notification
import github.Organization
import github.PaginatedList
import github.Plan
import github.Repository
import github.UserKey

from . import Consts


class AuthenticatedUser(github.GithubObject.CompletableGithubObject):
    """
    This class represents AuthenticatedUsers as returned by https://developer.github.com/v3/users/#get-the-authenticated-user

    An AuthenticatedUser object can be created by calling ``get_user()`` on a Github object.
    """

    def __repr__(self):
        return self.get__repr__({"login": self._login.value})

    @property
    def avatar_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._avatar_url)
        return self._avatar_url.value

    @property
    def bio(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._bio)
        return self._bio.value

    @property
    def blog(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._blog)
        return self._blog.value

    @property
    def collaborators(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._collaborators)
        return self._collaborators.value

    @property
    def company(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._company)
        return self._company.value

    @property
    def created_at(self):
        """
        :type: datetime.datetime
        """
        self._completeIfNotSet(self._created_at)
        return self._created_at.value

    @property
    def disk_usage(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._disk_usage)
        return self._disk_usage.value

    @property
    def email(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._email)
        return self._email.value

    @property
    def events_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._events_url)
        return self._events_url.value

    @property
    def followers(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._followers)
        return self._followers.value

    @property
    def followers_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._followers_url)
        return self._followers_url.value

    @property
    def following(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._following)
        return self._following.value

    @property
    def following_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._following_url)
        return self._following_url.value

    @property
    def gists_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._gists_url)
        return self._gists_url.value

    @property
    def gravatar_id(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._gravatar_id)
        return self._gravatar_id.value

    @property
    def hireable(self):
        """
        :type: bool
        """
        self._completeIfNotSet(self._hireable)
        return self._hireable.value

    @property
    def html_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._html_url)
        return self._html_url.value

    @property
    def id(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._id)
        return self._id.value

    @property
    def location(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._location)
        return self._location.value

    @property
    def login(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._login)
        return self._login.value

    @property
    def name(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._name)
        return self._name.value

    @property
    def node_id(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._node_id)
        return self._node_id.value

    @property
    def organizations_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._organizations_url)
        return self._organizations_url.value

    @property
    def owned_private_repos(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._owned_private_repos)
        return self._owned_private_repos.value

    @property
    def plan(self):
        """
        :type: :class:`github.Plan.Plan`
        """
        self._completeIfNotSet(self._plan)
        return self._plan.value

    @property
    def private_gists(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._private_gists)
        return self._private_gists.value

    @property
    def public_gists(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._public_gists)
        return self._public_gists.value

    @property
    def public_repos(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._public_repos)
        return self._public_repos.value

    @property
    def received_events_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._received_events_url)
        return self._received_events_url.value

    @property
    def repos_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._repos_url)
        return self._repos_url.value

    @property
    def site_admin(self):
        """
        :type: bool
        """
        self._completeIfNotSet(self._site_admin)
        return self._site_admin.value

    @property
    def starred_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._starred_url)
        return self._starred_url.value

    @property
    def subscriptions_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._subscriptions_url)
        return self._subscriptions_url.value

    @property
    def total_private_repos(self):
        """
        :type: integer
        """
        self._completeIfNotSet(self._total_private_repos)
        return self._total_private_repos.value

    @property
    def type(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._type)
        return self._type.value

    @property
    def updated_at(self):
        """
        :type: datetime.datetime
        """
        self._completeIfNotSet(self._updated_at)
        return self._updated_at.value

    @property
    def url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._url)
        return self._url.value

    def add_to_emails(self, *emails):
        """
        :calls: `POST /user/emails <http://developer.github.com/v3/users/emails>`_
        :param email: string
        :rtype: None
        """
        assert all(isinstance(element, str) for element in emails), emails
        post_parameters = emails
        headers, data = self._requester.requestJsonAndCheck(
            "POST", "/user/emails", input=post_parameters
        )

    def add_to_following(self, following):
        """
        :calls: `PUT /user/following/:user <http://developer.github.com/v3/users/followers>`_
        :param following: :class:`github.NamedUser.NamedUser`
        :rtype: None
        """
        assert isinstance(following, github.NamedUser.NamedUser), following
        headers, data = self._requester.requestJsonAndCheck(
            "PUT", "/user/following/" + following._identity
        )

    def add_to_starred(self, starred):
        """
        :calls: `PUT /user/starred/:owner/:repo <http://developer.github.com/v3/activity/starring>`_
        :param starred: :class:`github.Repository.Repository`
        :rtype: None
        """
        assert isinstance(starred, github.Repository.Repository), starred
        headers, data = self._requester.requestJsonAndCheck(
            "PUT", "/user/starred/" + starred._identity
        )

    def add_to_subscriptions(self, subscription):
        """
        :calls: `PUT /user/subscriptions/:owner/:repo <http://developer.github.com/v3/activity/watching>`_
        :param subscription: :class:`github.Repository.Repository`
        :rtype: None
        """
        assert isinstance(subscription, github.Repository.Repository), subscription
        headers, data = self._requester.requestJsonAndCheck(
            "PUT", "/user/subscriptions/" + subscription._identity
        )

    def add_to_watched(self, watched):
        """
        :calls: `PUT /repos/:owner/:repo/subscription <http://developer.github.com/v3/activity/watching>`_
        :param watched: :class:`github.Repository.Repository`
        :rtype: None
        """
        assert isinstance(watched, github.Repository.Repository), watched
        headers, data = self._requester.requestJsonAndCheck(
            "PUT",
            "/repos/" + watched._identity + "/subscription",
            input={"subscribed": True},
        )

    def create_authorization(
        self,
        scopes=github.GithubObject.NotSet,
        note=github.GithubObject.NotSet,
        note_url=github.GithubObject.NotSet,
        client_id=github.GithubObject.NotSet,
        client_secret=github.GithubObject.NotSet,
        onetime_password=None,
    ):
        """
        :calls: `POST /authorizations <http://developer.github.com/v3/oauth>`_
        :param scopes: list of string
        :param note: string
        :param note_url: string
        :param client_id: string
        :param client_secret: string
        :param onetime_password: string
        :rtype: :class:`github.Authorization.Authorization`
        """
        assert scopes is github.GithubObject.NotSet or all(
            isinstance(element, str) for element in scopes
        ), scopes
        assert note is github.GithubObject.NotSet or isinstance(note, str), note
        assert note_url is github.GithubObject.NotSet or isinstance(
            note_url, str
        ), note_url
        assert client_id is github.GithubObject.NotSet or isinstance(
            client_id, str
        ), client_id
        assert client_secret is github.GithubObject.NotSet or isinstance(
            client_secret, str
        ), client_secret
        assert onetime_password is None or isinstance(
            onetime_password, str
        ), onetime_password
        post_parameters = dict()
        if scopes is not github.GithubObject.NotSet:
            post_parameters["scopes"] = scopes
        if note is not github.GithubObject.NotSet:
            post_parameters["note"] = note
        if note_url is not github.GithubObject.NotSet:
            post_parameters["note_url"] = note_url
        if client_id is not github.GithubObject.NotSet:
            post_parameters["client_id"] = client_id
        if client_secret is not github.GithubObject.NotSet:
            post_parameters["client_secret"] = client_secret
        if onetime_password is not None:
            request_header = {
                Consts.headerOTP: onetime_password
            }  # pragma no cover (Should be covered)
        else:
            request_header = None
        headers, data = self._requester.requestJsonAndCheck(
            "POST", "/authorizations", input=post_parameters, headers=request_header,
        )
        return github.Authorization.Authorization(
            self._requester, headers, data, completed=True
        )

    def create_fork(self, repo):
        """
        :calls: `POST /repos/:owner/:repo/forks <http://developer.github.com/v3/repos/forks>`_
        :param repo: :class:`github.Repository.Repository`
        :rtype: :class:`github.Repository.Repository`
        """
        assert isinstance(repo, github.Repository.Repository), repo
        headers, data = self._requester.requestJsonAndCheck(
            "POST", "/repos/" + repo.owner.login + "/" + repo.name + "/forks"
        )
        return github.Repository.Repository(
            self._requester, headers, data, completed=True
        )

    def create_gist(self, public, files, description=github.GithubObject.NotSet):
        """
        :calls: `POST /gists <http://developer.github.com/v3/gists>`_
        :param public: bool
        :param files: dict of string to :class:`github.InputFileContent.InputFileContent`
        :param description: string
        :rtype: :class:`github.Gist.Gist`
        """
        assert isinstance(public, bool), public
        assert all(
            isinstance(element, github.InputFileContent) for element in files.values()
        ), files
        assert description is github.GithubObject.NotSet or isinstance(
            description, str
        ), description
        post_parameters = {
            "public": public,
            "files": {key: value._identity for key, value in files.items()},
        }
        if description is not github.GithubObject.NotSet:
            post_parameters["description"] = description
        headers, data = self._requester.requestJsonAndCheck(
            "POST", "/gists", input=post_parameters
        )
        return github.Gist.Gist(self._requester, headers, data, completed=True)

    def create_key(self, title, key):
        """
        :calls: `POST /user/keys <http://developer.github.com/v3/users/keys>`_
        :param title: string
        :param key: string
        :rtype: :class:`github.UserKey.UserKey`
        """
        assert isinstance(title, str), title
        assert isinstance(key, str), key
        post_parameters = {
            "title": title,
            "key": key,
        }
        headers, data = self._requester.requestJsonAndCheck(
            "POST", "/user/keys", input=post_parameters
        )
        return github.UserKey.UserKey(self._requester, headers, data, completed=True)

    def create_project(self, name, body=github.GithubObject.NotSet):
        """
        :calls: `POST /user/projects <https://developer.github.com/v3/projects/#create-a-user-project>`_
        :param name: string
        :param body: string
        :rtype: :class:`github.Project.Project`
        """
        assert isinstance(name, str), name
        assert body is github.GithubObject.NotSet or isinstance(body, str), body
        post_parameters = {
            "name": name,
            "body": body,
        }
        headers, data = self._requester.requestJsonAndCheck(
            "POST",
            "/user/projects",
            input=post_parameters,
            headers={"Accept": Consts.mediaTypeProjectsPreview},
        )
        return github.Project.Project(self._requester, headers, data, completed=True)

    def create_repo(
        self,
        name,
        description=github.GithubObject.NotSet,
        homepage=github.GithubObject.NotSet,
        private=github.GithubObject.NotSet,
        has_issues=github.GithubObject.NotSet,
        has_wiki=github.GithubObject.NotSet,
        has_downloads=github.GithubObject.NotSet,
        has_projects=github.GithubObject.NotSet,
        auto_init=github.GithubObject.NotSet,
        license_template=github.GithubObject.NotSet,
        gitignore_template=github.GithubObject.NotSet,
        allow_squash_merge=github.GithubObject.NotSet,
        allow_merge_commit=github.GithubObject.NotSet,
        allow_rebase_merge=github.GithubObject.NotSet,
        delete_branch_on_merge=github.GithubObject.NotSet,
    ):
        """
        :calls: `POST /user/repos <http://developer.github.com/v3/repos>`_
        :param name: string
        :param description: string
        :param homepage: string
        :param private: bool
        :param has_issues: bool
        :param has_wiki: bool
        :param has_downloads: bool
        :param has_projects: bool
        :param auto_init: bool
        :param license_template: string
        :param gitignore_template: string
        :param allow_squash_merge: bool
        :param allow_merge_commit: bool
        :param allow_rebase_merge: bool
        :param delete_branch_on_merge: bool
        :rtype: :class:`github.Repository.Repository`
        """
        assert isinstance(name, str), name
        assert description is github.GithubObject.NotSet or isinstance(
            description, str
        ), description
        assert homepage is github.GithubObject.NotSet or isinstance(
            homepage, str
        ), homepage
        assert private is github.GithubObject.NotSet or isinstance(
            private, bool
        ), private
        assert has_issues is github.GithubObject.NotSet or isinstance(
            has_issues, bool
        ), has_issues
        assert has_wiki is github.GithubObject.NotSet or isinstance(
            has_wiki, bool
        ), has_wiki
        assert has_downloads is github.GithubObject.NotSet or isinstance(
            has_downloads, bool
        ), has_downloads
        assert has_projects is github.GithubObject.NotSet or isinstance(
            has_projects, bool
        ), has_projects
        assert auto_init is github.GithubObject.NotSet or isinstance(
            auto_init, bool
        ), auto_init
        assert license_template is github.GithubObject.NotSet or isinstance(
            license_template, str
        ), license_template
        assert gitignore_template is github.GithubObject.NotSet or isinstance(
            gitignore_template, str
        ), gitignore_template
        assert allow_squash_merge is github.GithubObject.NotSet or isinstance(
            allow_squash_merge, bool
        ), allow_squash_merge
        assert allow_merge_commit is github.GithubObject.NotSet or isinstance(
            allow_merge_commit, bool
        ), allow_merge_commit
        assert allow_rebase_merge is github.GithubObject.NotSet or isinstance(
            allow_rebase_merge, bool
        ), allow_rebase_merge
        assert delete_branch_on_merge is github.GithubObject.NotSet or isinstance(
            delete_branch_on_merge, bool
        ), delete_branch_on_merge
        post_parameters = {
            "name": name,
        }
        if description is not github.GithubObject.NotSet:
            post_parameters["description"] = description
        if homepage is not github.GithubObject.NotSet:
            post_parameters["homepage"] = homepage
        if private is not github.GithubObject.NotSet:
            post_parameters["private"] = private
        if has_issues is not github.GithubObject.NotSet:
            post_parameters["has_issues"] = has_issues
        if has_wiki is not github.GithubObject.NotSet:
            post_parameters["has_wiki"] = has_wiki
        if has_downloads is not github.GithubObject.NotSet:
            post_parameters["has_downloads"] = has_downloads
        if has_projects is not github.GithubObject.NotSet:
            post_parameters["has_projects"] = has_projects
        if auto_init is not github.GithubObject.NotSet:
            post_parameters["auto_init"] = auto_init
        if license_template is not github.GithubObject.NotSet:
            post_parameters["license_template"] = license_template
        if gitignore_template is not github.GithubObject.NotSet:
            post_parameters["gitignore_template"] = gitignore_template
        if allow_squash_merge is not github.GithubObject.NotSet:
            post_parameters["allow_squash_merge"] = allow_squash_merge
        if allow_merge_commit is not github.GithubObject.NotSet:
            post_parameters["allow_merge_commit"] = allow_merge_commit
        if allow_rebase_merge is not github.GithubObject.NotSet:
            post_parameters["allow_rebase_merge"] = allow_rebase_merge
        if delete_branch_on_merge is not github.GithubObject.NotSet:
            post_parameters["delete_branch_on_merge"] = delete_branch_on_merge
        headers, data = self._requester.requestJsonAndCheck(
            "POST", "/user/repos", input=post_parameters
        )
        return github.Repository.Repository(
            self._requester, headers, data, completed=True
        )

    def edit(
        self,
        name=github.GithubObject.NotSet,
        email=github.GithubObject.NotSet,
        blog=github.GithubObject.NotSet,
        company=github.GithubObject.NotSet,
        location=github.GithubObject.NotSet,
        hireable=github.GithubObject.NotSet,
        bio=github.GithubObject.NotSet,
    ):
        """
        :calls: `PATCH /user <http://developer.github.com/v3/users>`_
        :param name: string
        :param email: string
        :param blog: string
        :param company: string
        :param location: string
        :param hireable: bool
        :param bio: string
        :rtype: None
        """
        assert name is github.GithubObject.NotSet or isinstance(name, str), name
        assert email is github.GithubObject.NotSet or isinstance(email, str), email
        assert blog is github.GithubObject.NotSet or isinstance(blog, str), blog
        assert company is github.GithubObject.NotSet or isinstance(
            company, str
        ), company
        assert location is github.GithubObject.NotSet or isinstance(
            location, str
        ), location
        assert hireable is github.GithubObject.NotSet or isinstance(
            hireable, bool
        ), hireable
        assert bio is github.GithubObject.NotSet or isinstance(bio, str), bio
        post_parameters = dict()
        if name is not github.GithubObject.NotSet:
            post_parameters["name"] = name
        if email is not github.GithubObject.NotSet:
            post_parameters["email"] = email
        if blog is not github.GithubObject.NotSet:
            post_parameters["blog"] = blog
        if company is not github.GithubObject.NotSet:
            post_parameters["company"] = company
        if location is not github.GithubObject.NotSet:
            post_parameters["location"] = location
        if hireable is not github.GithubObject.NotSet:
            post_parameters["hireable"] = hireable
        if bio is not github.GithubObject.NotSet:
            post_parameters["bio"] = bio
        headers, data = self._requester.requestJsonAndCheck(
            "PATCH", "/user", input=post_parameters
        )
        self._useAttributes(data)

    def get_authorization(self, id):
        """
        :calls: `GET /authorizations/:id <http://developer.github.com/v3/oauth>`_
        :param id: integer
        :rtype: :class:`github.Authorization.Authorization`
        """
        assert isinstance(id, int), id
        headers, data = self._requester.requestJsonAndCheck(
            "GET", "/authorizations/" + str(id)
        )
        return github.Authorization.Authorization(
            self._requester, headers, data, completed=True
        )

    def get_authorizations(self):
        """
        :calls: `GET /authorizations <http://developer.github.com/v3/oauth>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Authorization.Authorization`
        """
        return github.PaginatedList.PaginatedList(
            github.Authorization.Authorization, self._requester, "/authorizations", None
        )

    def get_emails(self):
        """
        :calls: `GET /user/emails <http://developer.github.com/v3/users/emails>`_
        :rtype: list of string
        """
        headers, data = self._requester.requestJsonAndCheck("GET", "/user/emails")
        return data

    def get_events(self):
        """
        :calls: `GET /events <http://developer.github.com/v3/activity/events>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Event.Event`
        """
        return github.PaginatedList.PaginatedList(
            github.Event.Event, self._requester, "/events", None
        )

    def get_followers(self):
        """
        :calls: `GET /user/followers <http://developer.github.com/v3/users/followers>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.NamedUser.NamedUser`
        """
        return github.PaginatedList.PaginatedList(
            github.NamedUser.NamedUser, self._requester, "/user/followers", None
        )

    def get_following(self):
        """
        :calls: `GET /user/following <http://developer.github.com/v3/users/followers>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.NamedUser.NamedUser`
        """
        return github.PaginatedList.PaginatedList(
            github.NamedUser.NamedUser, self._requester, "/user/following", None
        )

    def get_gists(self, since=github.GithubObject.NotSet):
        """
        :calls: `GET /gists <http://developer.github.com/v3/gists>`_
        :param since: datetime.datetime format YYYY-MM-DDTHH:MM:SSZ
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Gist.Gist`
        """
        assert since is github.GithubObject.NotSet or isinstance(
            since, datetime.datetime
        ), since
        url_parameters = dict()
        if since is not github.GithubObject.NotSet:
            url_parameters["since"] = since.strftime("%Y-%m-%dT%H:%M:%SZ")
        return github.PaginatedList.PaginatedList(
            github.Gist.Gist, self._requester, "/gists", url_parameters
        )

    def get_issues(
        self,
        filter=github.GithubObject.NotSet,
        state=github.GithubObject.NotSet,
        labels=github.GithubObject.NotSet,
        sort=github.GithubObject.NotSet,
        direction=github.GithubObject.NotSet,
        since=github.GithubObject.NotSet,
    ):
        """
        :calls: `GET /issues <http://developer.github.com/v3/issues>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Issue.Issue`
        :param filter: string
        :param state: string
        :param labels: list of :class:`github.Label.Label`
        :param sort: string
        :param direction: string
        :param since: datetime.datetime
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Issue.Issue`
        """
        assert filter is github.GithubObject.NotSet or isinstance(filter, str), filter
        assert state is github.GithubObject.NotSet or isinstance(state, str), state
        assert labels is github.GithubObject.NotSet or all(
            isinstance(element, github.Label.Label) for element in labels
        ), labels
        assert sort is github.GithubObject.NotSet or isinstance(sort, str), sort
        assert direction is github.GithubObject.NotSet or isinstance(
            direction, str
        ), direction
        assert since is github.GithubObject.NotSet or isinstance(
            since, datetime.datetime
        ), since
        url_parameters = dict()
        if filter is not github.GithubObject.NotSet:
            url_parameters["filter"] = filter
        if state is not github.GithubObject.NotSet:
            url_parameters["state"] = state
        if labels is not github.GithubObject.NotSet:
            url_parameters["labels"] = ",".join(label.name for label in labels)
        if sort is not github.GithubObject.NotSet:
            url_parameters["sort"] = sort
        if direction is not github.GithubObject.NotSet:
            url_parameters["direction"] = direction
        if since is not github.GithubObject.NotSet:
            url_parameters["since"] = since.strftime("%Y-%m-%dT%H:%M:%SZ")
        return github.PaginatedList.PaginatedList(
            github.Issue.Issue, self._requester, "/issues", url_parameters
        )

    def get_user_issues(
        self,
        filter=github.GithubObject.NotSet,
        state=github.GithubObject.NotSet,
        labels=github.GithubObject.NotSet,
        sort=github.GithubObject.NotSet,
        direction=github.GithubObject.NotSet,
        since=github.GithubObject.NotSet,
    ):
        """
        :calls: `GET /user/issues <http://developer.github.com/v3/issues>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Issue.Issue`
        :param filter: string
        :param state: string
        :param labels: list of :class:`github.Label.Label`
        :param sort: string
        :param direction: string
        :param since: datetime.datetime
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Issue.Issue`
        """
        assert filter is github.GithubObject.NotSet or isinstance(filter, str), filter
        assert state is github.GithubObject.NotSet or isinstance(state, str), state
        assert labels is github.GithubObject.NotSet or all(
            isinstance(element, github.Label.Label) for element in labels
        ), labels
        assert sort is github.GithubObject.NotSet or isinstance(sort, str), sort
        assert direction is github.GithubObject.NotSet or isinstance(
            direction, str
        ), direction
        assert since is github.GithubObject.NotSet or isinstance(
            since, datetime.datetime
        ), since
        url_parameters = dict()
        if filter is not github.GithubObject.NotSet:
            url_parameters["filter"] = filter
        if state is not github.GithubObject.NotSet:
            url_parameters["state"] = state
        if labels is not github.GithubObject.NotSet:
            url_parameters["labels"] = ",".join(label.name for label in labels)
        if sort is not github.GithubObject.NotSet:
            url_parameters["sort"] = sort
        if direction is not github.GithubObject.NotSet:
            url_parameters["direction"] = direction
        if since is not github.GithubObject.NotSet:
            url_parameters["since"] = since.strftime("%Y-%m-%dT%H:%M:%SZ")
        return github.PaginatedList.PaginatedList(
            github.Issue.Issue, self._requester, "/issues", url_parameters
        )

    def get_key(self, id):
        """
        :calls: `GET /user/keys/:id <http://developer.github.com/v3/users/keys>`_
        :param id: integer
        :rtype: :class:`github.UserKey.UserKey`
        """
        assert isinstance(id, int), id
        headers, data = self._requester.requestJsonAndCheck(
            "GET", "/user/keys/" + str(id)
        )
        return github.UserKey.UserKey(self._requester, headers, data, completed=True)

    def get_keys(self):
        """
        :calls: `GET /user/keys <http://developer.github.com/v3/users/keys>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.UserKey.UserKey`
        """
        return github.PaginatedList.PaginatedList(
            github.UserKey.UserKey, self._requester, "/user/keys", None
        )

    def get_notification(self, id):
        """
        :calls: `GET /notifications/threads/:id <http://developer.github.com/v3/activity/notifications>`_
        :rtype: :class:`github.Notification.Notification`
        """

        assert isinstance(id, str), id
        headers, data = self._requester.requestJsonAndCheck(
            "GET", "/notifications/threads/" + id
        )
        return github.Notification.Notification(
            self._requester, headers, data, completed=True
        )

    def get_notifications(
        self,
        all=github.GithubObject.NotSet,
        participating=github.GithubObject.NotSet,
        since=github.GithubObject.NotSet,
        before=github.GithubObject.NotSet,
    ):
        """
        :calls: `GET /notifications <http://developer.github.com/v3/activity/notifications>`_
        :param all: bool
        :param participating: bool
        :param since: datetime.datetime
        :param before: datetime.datetime
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Notification.Notification`
        """

        assert all is github.GithubObject.NotSet or isinstance(all, bool), all
        assert participating is github.GithubObject.NotSet or isinstance(
            participating, bool
        ), participating
        assert since is github.GithubObject.NotSet or isinstance(
            since, datetime.datetime
        ), since
        assert before is github.GithubObject.NotSet or isinstance(
            before, datetime.datetime
        ), before

        params = dict()
        if all is not github.GithubObject.NotSet:
            params["all"] = all
        if participating is not github.GithubObject.NotSet:
            params["participating"] = participating
        if since is not github.GithubObject.NotSet:
            params["since"] = since.strftime("%Y-%m-%dT%H:%M:%SZ")
        if before is not github.GithubObject.NotSet:
            params["before"] = before.strftime("%Y-%m-%dT%H:%M:%SZ")

        return github.PaginatedList.PaginatedList(
            github.Notification.Notification, self._requester, "/notifications", params
        )

    def get_organization_events(self, org):
        """
        :calls: `GET /users/:user/events/orgs/:org <http://developer.github.com/v3/activity/events>`_
        :param org: :class:`github.Organization.Organization`
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Event.Event`
        """
        assert isinstance(org, github.Organization.Organization), org
        return github.PaginatedList.PaginatedList(
            github.Event.Event,
            self._requester,
            "/users/" + self.login + "/events/orgs/" + org.login,
            None,
        )

    def get_orgs(self):
        """
        :calls: `GET /user/orgs <http://developer.github.com/v3/orgs>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Organization.Organization`
        """
        return github.PaginatedList.PaginatedList(
            github.Organization.Organization, self._requester, "/user/orgs", None
        )

    def get_repo(self, name):
        """
        :calls: `GET /repos/:owner/:repo <http://developer.github.com/v3/repos>`_
        :param name: string
        :rtype: :class:`github.Repository.Repository`
        """
        assert isinstance(name, str), name
        headers, data = self._requester.requestJsonAndCheck(
            "GET", "/repos/" + self.login + "/" + name
        )
        return github.Repository.Repository(
            self._requester, headers, data, completed=True
        )

    def get_repos(
        self,
        visibility=github.GithubObject.NotSet,
        affiliation=github.GithubObject.NotSet,
        type=github.GithubObject.NotSet,
        sort=github.GithubObject.NotSet,
        direction=github.GithubObject.NotSet,
    ):
        """
        :calls: `GET /user/repos <http://developer.github.com/v3/repos>`
        :param visibility: string
        :param affiliation: string
        :param type: string
        :param sort: string
        :param direction: string
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Repository.Repository`
        """
        assert visibility is github.GithubObject.NotSet or isinstance(
            visibility, str
        ), visibility
        assert affiliation is github.GithubObject.NotSet or isinstance(
            affiliation, str
        ), affiliation
        assert type is github.GithubObject.NotSet or isinstance(type, str), type
        assert sort is github.GithubObject.NotSet or isinstance(sort, str), sort
        assert direction is github.GithubObject.NotSet or isinstance(
            direction, str
        ), direction
        url_parameters = dict()
        if visibility is not github.GithubObject.NotSet:
            url_parameters["visibility"] = visibility
        if affiliation is not github.GithubObject.NotSet:
            url_parameters["affiliation"] = affiliation
        if type is not github.GithubObject.NotSet:
            url_parameters["type"] = type
        if sort is not github.GithubObject.NotSet:
            url_parameters["sort"] = sort
        if direction is not github.GithubObject.NotSet:
            url_parameters["direction"] = direction
        return github.PaginatedList.PaginatedList(
            github.Repository.Repository, self._requester, "/user/repos", url_parameters
        )

    def get_starred(self):
        """
        :calls: `GET /user/starred <http://developer.github.com/v3/activity/starring>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Repository.Repository`
        """
        return github.PaginatedList.PaginatedList(
            github.Repository.Repository, self._requester, "/user/starred", None
        )

    def get_starred_gists(self):
        """
        :calls: `GET /gists/starred <http://developer.github.com/v3/gists>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Gist.Gist`
        """
        return github.PaginatedList.PaginatedList(
            github.Gist.Gist, self._requester, "/gists/starred", None
        )

    def get_subscriptions(self):
        """
        :calls: `GET /user/subscriptions <http://developer.github.com/v3/activity/watching>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Repository.Repository`
        """
        return github.PaginatedList.PaginatedList(
            github.Repository.Repository, self._requester, "/user/subscriptions", None
        )

    def get_teams(self):
        """
        :calls: `GET /user/teams <http://developer.github.com/v3/orgs/teams>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Team.Team`
        """
        return github.PaginatedList.PaginatedList(
            github.Team.Team, self._requester, "/user/teams", None
        )

    def get_watched(self):
        """
        :calls: `GET /user/subscriptions <http://developer.github.com/v3/activity/watching>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Repository.Repository`
        """
        return github.PaginatedList.PaginatedList(
            github.Repository.Repository, self._requester, "/user/subscriptions", None
        )

    def get_installations(self):
        """
        :calls: `GET /user/installations <http://developer.github.com/v3/apps>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Installation.Installation`
        """
        return github.PaginatedList.PaginatedList(
            github.Installation.Installation,
            self._requester,
            "/user/installations",
            None,
            headers={"Accept": Consts.mediaTypeIntegrationPreview},
            list_item="installations",
        )

    def has_in_following(self, following):
        """
        :calls: `GET /user/following/:user <http://developer.github.com/v3/users/followers>`_
        :param following: :class:`github.NamedUser.NamedUser`
        :rtype: bool
        """
        assert isinstance(following, github.NamedUser.NamedUser), following
        status, headers, data = self._requester.requestJson(
            "GET", "/user/following/" + following._identity
        )
        return status == 204

    def has_in_starred(self, starred):
        """
        :calls: `GET /user/starred/:owner/:repo <http://developer.github.com/v3/activity/starring>`_
        :param starred: :class:`github.Repository.Repository`
        :rtype: bool
        """
        assert isinstance(starred, github.Repository.Repository), starred
        status, headers, data = self._requester.requestJson(
            "GET", "/user/starred/" + starred._identity
        )
        return status == 204

    def has_in_subscriptions(self, subscription):
        """
        :calls: `GET /user/subscriptions/:owner/:repo <http://developer.github.com/v3/activity/watching>`_
        :param subscription: :class:`github.Repository.Repository`
        :rtype: bool
        """
        assert isinstance(subscription, github.Repository.Repository), subscription
        status, headers, data = self._requester.requestJson(
            "GET", "/user/subscriptions/" + subscription._identity
        )
        return status == 204

    def has_in_watched(self, watched):
        """
        :calls: `GET /repos/:owner/:repo/subscription <http://developer.github.com/v3/activity/watching>`_
        :param watched: :class:`github.Repository.Repository`
        :rtype: bool
        """
        assert isinstance(watched, github.Repository.Repository), watched
        status, headers, data = self._requester.requestJson(
            "GET", "/repos/" + watched._identity + "/subscription"
        )
        return status == 200

    def mark_notifications_as_read(self, last_read_at=datetime.datetime.utcnow()):
        """
        :calls: `PUT /notifications <https://developer.github.com/v3/activity/notifications>`_
        :param last_read_at: datetime
        """
        assert isinstance(last_read_at, datetime.datetime)
        put_parameters = {"last_read_at": last_read_at.strftime("%Y-%m-%dT%H:%M:%SZ")}

        headers, data = self._requester.requestJsonAndCheck(
            "PUT", "/notifications", input=put_parameters
        )

    def remove_from_emails(self, *emails):
        """
        :calls: `DELETE /user/emails <http://developer.github.com/v3/users/emails>`_
        :param email: string
        :rtype: None
        """
        assert all(isinstance(element, str) for element in emails), emails
        post_parameters = emails
        headers, data = self._requester.requestJsonAndCheck(
            "DELETE", "/user/emails", input=post_parameters
        )

    def remove_from_following(self, following):
        """
        :calls: `DELETE /user/following/:user <http://developer.github.com/v3/users/followers>`_
        :param following: :class:`github.NamedUser.NamedUser`
        :rtype: None
        """
        assert isinstance(following, github.NamedUser.NamedUser), following
        headers, data = self._requester.requestJsonAndCheck(
            "DELETE", "/user/following/" + following._identity
        )

    def remove_from_starred(self, starred):
        """
        :calls: `DELETE /user/starred/:owner/:repo <http://developer.github.com/v3/activity/starring>`_
        :param starred: :class:`github.Repository.Repository`
        :rtype: None
        """
        assert isinstance(starred, github.Repository.Repository), starred
        headers, data = self._requester.requestJsonAndCheck(
            "DELETE", "/user/starred/" + starred._identity
        )

    def remove_from_subscriptions(self, subscription):
        """
        :calls: `DELETE /user/subscriptions/:owner/:repo <http://developer.github.com/v3/activity/watching>`_
        :param subscription: :class:`github.Repository.Repository`
        :rtype: None
        """
        assert isinstance(subscription, github.Repository.Repository), subscription
        headers, data = self._requester.requestJsonAndCheck(
            "DELETE", "/user/subscriptions/" + subscription._identity
        )

    def remove_from_watched(self, watched):
        """
        :calls: `DELETE /repos/:owner/:repo/subscription <http://developer.github.com/v3/activity/watching>`_
        :param watched: :class:`github.Repository.Repository`
        :rtype: None
        """
        assert isinstance(watched, github.Repository.Repository), watched
        headers, data = self._requester.requestJsonAndCheck(
            "DELETE", "/repos/" + watched._identity + "/subscription"
        )

    def accept_invitation(self, invitation):
        """
        :calls: `PATCH /user/repository_invitations/:invitation_id <https://developer.github.com/v3/repos/invitations/>`
        :param invitation: :class:`github.Invitation.Invitation` or int
        :rtype: None
        """
        assert isinstance(invitation, github.Invitation.Invitation) or isinstance(
            invitation, int
        )

        if isinstance(invitation, github.Invitation.Invitation):
            invitation = invitation.id

        headers, data = self._requester.requestJsonAndCheck(
            "PATCH", "/user/repository_invitations/" + str(invitation), input={}
        )

    def get_invitations(self):
        """
        :calls: `GET /user/repository_invitations <https://developer.github.com/v3/repos/invitations/>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Invitation.Invitation`
        """
        return github.PaginatedList.PaginatedList(
            github.Invitation.Invitation,
            self._requester,
            "/user/repository_invitations",
            None,
        )

    def create_migration(
        self,
        repos,
        lock_repositories=github.GithubObject.NotSet,
        exclude_attachments=github.GithubObject.NotSet,
    ):
        """
        :calls: `POST /user/migrations <https://developer.github.com/v3/migrations/users>`_
        :param repos: list or tuple of str
        :param lock_repositories: bool
        :param exclude_attachments: bool
        :rtype: :class:`github.Migration.Migration`
        """
        assert isinstance(repos, (list, tuple)), repos
        assert all(isinstance(repo, str) for repo in repos), repos
        assert lock_repositories is github.GithubObject.NotSet or isinstance(
            lock_repositories, bool
        ), lock_repositories
        assert exclude_attachments is github.GithubObject.NotSet or isinstance(
            exclude_attachments, bool
        ), exclude_attachments
        post_parameters = {"repositories": repos}
        if lock_repositories is not github.GithubObject.NotSet:
            post_parameters["lock_repositories"] = lock_repositories
        if exclude_attachments is not github.GithubObject.NotSet:
            post_parameters["exclude_attachments"] = exclude_attachments
        headers, data = self._requester.requestJsonAndCheck(
            "POST",
            "/user/migrations",
            input=post_parameters,
            headers={"Accept": Consts.mediaTypeMigrationPreview},
        )
        return github.Migration.Migration(
            self._requester, headers, data, completed=True
        )

    def get_migrations(self):
        """
        :calls: `GET /user/migrations <https://developer.github.com/v3/migrations/users>`_
        :rtype: :class:`github.PaginatedList.PaginatedList` of :class:`github.Migration.Migration`
        """
        return github.PaginatedList.PaginatedList(
            github.Migration.Migration,
            self._requester,
            "/user/migrations",
            None,
            headers={"Accept": Consts.mediaTypeMigrationPreview},
        )

    def get_organization_membership(self, org):
        """
        :calls: `GET /user/memberships/orgs/:org <https://developer.github.com/v3/orgs/members/#get-your-organization-membership>`_
        :rtype: :class:`github.Membership.Membership`
        """
        assert isinstance(org, str)
        headers, data = self._requester.requestJsonAndCheck(
            "GET", "/user/memberships/orgs/" + org
        )
        return github.Membership.Membership(
            self._requester, headers, data, completed=True
        )

    def _initAttributes(self):
        self._avatar_url = github.GithubObject.NotSet
        self._bio = github.GithubObject.NotSet
        self._blog = github.GithubObject.NotSet
        self._collaborators = github.GithubObject.NotSet
        self._company = github.GithubObject.NotSet
        self._created_at = github.GithubObject.NotSet
        self._disk_usage = github.GithubObject.NotSet
        self._email = github.GithubObject.NotSet
        self._events_url = github.GithubObject.NotSet
        self._followers = github.GithubObject.NotSet
        self._followers_url = github.GithubObject.NotSet
        self._following = github.GithubObject.NotSet
        self._following_url = github.GithubObject.NotSet
        self._gists_url = github.GithubObject.NotSet
        self._gravatar_id = github.GithubObject.NotSet
        self._hireable = github.GithubObject.NotSet
        self._html_url = github.GithubObject.NotSet
        self._id = github.GithubObject.NotSet
        self._location = github.GithubObject.NotSet
        self._login = github.GithubObject.NotSet
        self._name = github.GithubObject.NotSet
        self._node_id = github.GithubObject.NotSet
        self._organizations_url = github.GithubObject.NotSet
        self._owned_private_repos = github.GithubObject.NotSet
        self._plan = github.GithubObject.NotSet
        self._private_gists = github.GithubObject.NotSet
        self._public_gists = github.GithubObject.NotSet
        self._public_repos = github.GithubObject.NotSet
        self._received_events_url = github.GithubObject.NotSet
        self._repos_url = github.GithubObject.NotSet
        self._site_admin = github.GithubObject.NotSet
        self._starred_url = github.GithubObject.NotSet
        self._subscriptions_url = github.GithubObject.NotSet
        self._total_private_repos = github.GithubObject.NotSet
        self._type = github.GithubObject.NotSet
        self._updated_at = github.GithubObject.NotSet
        self._url = github.GithubObject.NotSet

    def _useAttributes(self, attributes):
        if "avatar_url" in attributes:  # pragma no branch
            self._avatar_url = self._makeStringAttribute(attributes["avatar_url"])
        if "bio" in attributes:  # pragma no branch
            self._bio = self._makeStringAttribute(attributes["bio"])
        if "blog" in attributes:  # pragma no branch
            self._blog = self._makeStringAttribute(attributes["blog"])
        if "collaborators" in attributes:  # pragma no branch
            self._collaborators = self._makeIntAttribute(attributes["collaborators"])
        if "company" in attributes:  # pragma no branch
            self._company = self._makeStringAttribute(attributes["company"])
        if "created_at" in attributes:  # pragma no branch
            self._created_at = self._makeDatetimeAttribute(attributes["created_at"])
        if "disk_usage" in attributes:  # pragma no branch
            self._disk_usage = self._makeIntAttribute(attributes["disk_usage"])
        if "email" in attributes:  # pragma no branch
            self._email = self._makeStringAttribute(attributes["email"])
        if "events_url" in attributes:  # pragma no branch
            self._events_url = self._makeStringAttribute(attributes["events_url"])
        if "followers" in attributes:  # pragma no branch
            self._followers = self._makeIntAttribute(attributes["followers"])
        if "followers_url" in attributes:  # pragma no branch
            self._followers_url = self._makeStringAttribute(attributes["followers_url"])
        if "following" in attributes:  # pragma no branch
            self._following = self._makeIntAttribute(attributes["following"])
        if "following_url" in attributes:  # pragma no branch
            self._following_url = self._makeStringAttribute(attributes["following_url"])
        if "gists_url" in attributes:  # pragma no branch
            self._gists_url = self._makeStringAttribute(attributes["gists_url"])
        if "gravatar_id" in attributes:  # pragma no branch
            self._gravatar_id = self._makeStringAttribute(attributes["gravatar_id"])
        if "hireable" in attributes:  # pragma no branch
            self._hireable = self._makeBoolAttribute(attributes["hireable"])
        if "html_url" in attributes:  # pragma no branch
            self._html_url = self._makeStringAttribute(attributes["html_url"])
        if "id" in attributes:  # pragma no branch
            self._id = self._makeIntAttribute(attributes["id"])
        if "location" in attributes:  # pragma no branch
            self._location = self._makeStringAttribute(attributes["location"])
        if "login" in attributes:  # pragma no branch
            self._login = self._makeStringAttribute(attributes["login"])
        if "name" in attributes:  # pragma no branch
            self._name = self._makeStringAttribute(attributes["name"])
        if "node_id" in attributes:  # pragma no branch
            self._node_id = self._makeStringAttribute(attributes["node_id"])
        if "organizations_url" in attributes:  # pragma no branch
            self._organizations_url = self._makeStringAttribute(
                attributes["organizations_url"]
            )
        if "owned_private_repos" in attributes:  # pragma no branch
            self._owned_private_repos = self._makeIntAttribute(
                attributes["owned_private_repos"]
            )
        if "plan" in attributes:  # pragma no branch
            self._plan = self._makeClassAttribute(github.Plan.Plan, attributes["plan"])
        if "private_gists" in attributes:  # pragma no branch
            self._private_gists = self._makeIntAttribute(attributes["private_gists"])
        if "public_gists" in attributes:  # pragma no branch
            self._public_gists = self._makeIntAttribute(attributes["public_gists"])
        if "public_repos" in attributes:  # pragma no branch
            self._public_repos = self._makeIntAttribute(attributes["public_repos"])
        if "received_events_url" in attributes:  # pragma no branch
            self._received_events_url = self._makeStringAttribute(
                attributes["received_events_url"]
            )
        if "repos_url" in attributes:  # pragma no branch
            self._repos_url = self._makeStringAttribute(attributes["repos_url"])
        if "site_admin" in attributes:  # pragma no branch
            self._site_admin = self._makeBoolAttribute(attributes["site_admin"])
        if "starred_url" in attributes:  # pragma no branch
            self._starred_url = self._makeStringAttribute(attributes["starred_url"])
        if "subscriptions_url" in attributes:  # pragma no branch
            self._subscriptions_url = self._makeStringAttribute(
                attributes["subscriptions_url"]
            )
        if "total_private_repos" in attributes:  # pragma no branch
            self._total_private_repos = self._makeIntAttribute(
                attributes["total_private_repos"]
            )
        if "type" in attributes:  # pragma no branch
            self._type = self._makeStringAttribute(attributes["type"])
        if "updated_at" in attributes:  # pragma no branch
            self._updated_at = self._makeDatetimeAttribute(attributes["updated_at"])
        if "url" in attributes:  # pragma no branch
            self._url = self._makeStringAttribute(attributes["url"])



# -*- coding: utf-8 -*-

############################ Copyrights and license ###########################
#                                                                             #
# Copyright 2019 Rigas Papathanasopoulos <rigaspapas@gmail.com>               #
#                                                                             #
# This file is part of PyGithub.                                              #
# http://pygithub.readthedocs.io/                                             #
#                                                                             #
# PyGithub is free software: you can redistribute it and/or modify it under   #
# the terms of the GNU Lesser General Public License as published by the Free #
# Software Foundation, either version 3 of the License, or (at your option)   #
# any later version.                                                          #
#                                                                             #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS   #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more#
# details.                                                                    #
#                                                                             #
# You should have received a copy of the GNU Lesser General Public License    #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.            #
#                                                                             #
###############################################################################

import urllib

import github.GithubObject
from github.AccessToken import AccessToken


class ApplicationOAuth(github.GithubObject.NonCompletableGithubObject):
    """
    This class is used for identifying and authorizing users for Github Apps.
    https://developer.github.com/apps/building-github-apps/identifying-and-authorizing-users-for-github-apps/#1-users-are-redirected-to-request-their-github-identity
    """

    def __repr__(self):
        return self.get__repr__({"client_id": self._client_id.value})

    @property
    def client_id(self):
        return self._client_id.value

    @property
    def client_secret(self):
        return self._client_secret.value

    def _initAttributes(self):
        self._client_id = github.GithubObject.NotSet
        self._client_secret = github.GithubObject.NotSet

    def _useAttributes(self, attributes):
        if "client_id" in attributes:  # pragma no branch
            self._client_id = self._makeStringAttribute(attributes["client_id"])
        if "client_secret" in attributes:  # pragma no branch
            self._client_secret = self._makeStringAttribute(attributes["client_secret"])

    def get_login_url(self, redirect_uri=None, state=None, login=None):
        """
        Return the URL you need to redirect a user to in order to authorize
        your App.
        :type: string
        """
        parameters = {"client_id": self.client_id}
        if redirect_uri is not None:
            assert isinstance(redirect_uri, str), redirect_uri
            parameters["redirect_uri"] = redirect_uri
        if state is not None:
            assert isinstance(state, str), state
            parameters["state"] = state
        if login is not None:
            assert isinstance(login, str), login
            parameters["login"] = login

        parameters = urllib.parse.urlencode(parameters)

        base_url = "https://github.com/login/oauth/authorize"
        return u"{}?{}".format(base_url, parameters)

    def get_access_token(self, code, state=None):
        """
        :calls: `POST /login/oauth/access_token <https://developer.github.com/apps/building-github-apps/identifying-and-authorizing-users-for-github-apps/>`_
        :param code: string
        :param state: string
        """
        assert isinstance(code, str), code
        post_parameters = {
            "code": code,
            "client_id": self.client_id,
            "client_secret": self.client_secret,
        }

        if state is not None:
            post_parameters["state"] = state

        self._requester._Requester__authorizationHeader = None
        headers, data = self._requester.requestJsonAndCheck(
            "POST",
            "https://github.com/login/oauth/access_token",
            headers={
                "Accept": "application/json",
                "Content-Type": "application/json",
                "User-Agent": "PyGithub/Python",
            },
            input=post_parameters,
        )

        return AccessToken(
            requester=self._requester,
            # not required, this is a NonCompletableGithubObject
            headers={},
            attributes=data,
            completed=False,
        )



from typing import Any, Dict, Union

from github.GithubObject import NonCompletableGithubObject

class Plan(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def collaborators(self) -> int: ...
    @property
    def name(self) -> str: ...
    @property
    def private_repos(self) -> int: ...
    @property
    def space(self) -> int: ...



from datetime import datetime
from io import BytesIO
from typing import Any, Dict, List, Optional, Tuple, Type, TypeVar, Union, overload

from github.AuthenticatedUser import AuthenticatedUser
from github.Commit import Commit
from github.ContentFile import ContentFile
from github.Event import Event
from github.Gist import Gist
from github.GithubObject import GithubObject, _NotSetType
from github.GitignoreTemplate import GitignoreTemplate
from github.HookDescription import HookDescription
from github.Installation import Installation
from github.InstallationAuthorization import InstallationAuthorization
from github.Issue import Issue
from github.License import License
from github.NamedUser import NamedUser
from github.Organization import Organization
from github.PaginatedList import PaginatedList
from github.Project import Project
from github.RateLimit import RateLimit
from github.Repository import Repository
from github.Topic import Topic

# from urllib3.util.retry import Retry

TGithubObject = TypeVar('TGithubObject', bound=GithubObject)

class Github:
    def __init__(
        self,
        login_or_token: Optional[str] = ...,
        password: Optional[str] = ...,
        jwt: Optional[str] = ...,
        base_url: str = ...,
        timeout: int = ...,
        client_id: Optional[str] = ...,
        client_secret: Optional[str] = ...,
        user_agent: str = ...,
        per_page: int = ...,
        verify: bool = ...,
        retry: Any = ...,
    ) -> None: ...
    def __get_FIX_REPO_GET_GIT_REF(self) -> bool: ...
    def __set_FIX_REPO_GET_GIT_REF(self, value: bool) -> None: ...
    def __get_per_page(self) -> int: ...
    def __set_per_page(self, value: int) -> None: ...
    def create_from_raw_data(
        self,
        klass: Type[TGithubObject],
        raw_data: Dict[str, Any],
        headers: Dict[str, Union[str, int]] = ...,
    ) -> TGithubObject: ...
    def dump(self, obj: GithubObject, file: BytesIO, protocol: int = ...) -> None: ...
    def get_emojis(self) -> Dict[str, str]: ...
    def get_events(self) -> PaginatedList[Event]: ...
    def get_gist(self, id: str) -> Gist: ...
    def get_gists(
        self, since: Union[datetime, _NotSetType] = ...
    ) -> PaginatedList[Gist]: ...
    def get_gitignore_template(self, name: str) -> GitignoreTemplate: ...
    def get_gitignore_templates(self) -> List[str]: ...
    def get_hook(self, name: str) -> HookDescription: ...
    def get_hooks(self) -> List[HookDescription]: ...
    def get_installation(self, id: int) -> Installation: ...
    def get_license(self, key: Union[str, _NotSetType] = ...) -> License: ...
    def get_licenses(self) -> PaginatedList[License]: ...
    def get_organization(self, login: str) -> Organization: ...
    def get_organizations(
        self, since: Union[int, _NotSetType] = ...
    ) -> PaginatedList[Organization]: ...
    def get_project(self, id: int) -> Project: ...
    def get_rate_limit(self) -> RateLimit: ...
    def get_repo(
        self, full_name_or_id: Union[int, str], lazy: bool = ...
    ) -> Repository: ...
    def get_repos(
        self,
        since: Union[int, _NotSetType] = ...,
        visibility: Union[str, _NotSetType] = ...,
    ) -> PaginatedList[Repository]: ...
    @overload
    def get_user(self, login: _NotSetType) -> AuthenticatedUser: ...
    @overload
    def get_user(self, login: str) -> NamedUser: ...
    def get_users(
        self, since: Union[int, _NotSetType] = ...
    ) -> PaginatedList[NamedUser]: ...
    def load(self, f: BytesIO) -> Repository: ...
    @property
    def oauth_scopes(self) -> Optional[List[str]]: ...
    @property
    def rate_limiting(self) -> Tuple[int, int]: ...
    @property
    def rate_limiting_resettime(self) -> int: ...
    def render_markdown(
        self, text: str, context: Union[Repository, _NotSetType] = ...
    ) -> str: ...
    def search_code(
        self,
        query: str,
        sort: Union[str, _NotSetType] = ...,
        order: Union[str, _NotSetType] = ...,
        highlight: bool = ...,
        **qualifiers: Dict[str, Any]
    ) -> PaginatedList[ContentFile]: ...
    def search_commits(
        self,
        query: str,
        sort: Union[str, _NotSetType] = ...,
        order: Union[str, _NotSetType] = ...,
        **qualifiers: Dict[str, Any]
    ) -> PaginatedList[Commit]: ...
    def search_issues(
        self,
        query: str,
        sort: Union[str, _NotSetType] = ...,
        order: Union[str, _NotSetType] = ...,
        **qualifiers: Dict[str, Any]
    ) -> PaginatedList[Issue]: ...
    def search_repositories(
        self,
        query: str,
        sort: Union[str, _NotSetType] = ...,
        order: Union[str, _NotSetType] = ...,
        **qualifiers: Dict[str, Any]
    ) -> PaginatedList[Repository]: ...
    def search_topics(
        self,
        query: str,
        **qualifiers: Dict[str, Any]
    ) -> PaginatedList[Topic]: ...
    def search_users(
        self,
        query: str,
        sort: Union[str, _NotSetType] = ...,
        order: Union[str, _NotSetType] = ...,
        **qualifiers: Dict[str, Any]
    ) -> PaginatedList[NamedUser]: ...

class GithubIntegration:
    def __init__(
        self, integration_id: Union[int, str], private_key: str, base_url: str = ...
    ) -> None: ...
    def create_jwt(self, expiration: int = ...) -> str: ...
    def get_access_token(
        self, installation_id: int, user_id: Optional[int] = ...
    ) -> InstallationAuthorization: ...
    def get_installation(self, owner: str, repo: str) -> Installation: ...



# -*- coding: utf-8 -*-

############################ Copyrights and license ############################
#                                                                              #
# Copyright 2012 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2012 Zearin <zearin@gonk.net>                                      #
# Copyright 2013 AKFish <akfish@gmail.com>                                     #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Jannis Gebauer <ja.geb@me.com>                                #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import github.GitAuthor
import github.GithubObject
import github.GitTree


class GitCommit(github.GithubObject.CompletableGithubObject):
    """
    This class represents GitCommits. The reference can be found here https://developer.github.com/v3/git/commits/
    """

    def __repr__(self):
        return self.get__repr__({"sha": self._sha.value})

    @property
    def author(self):
        """
        :type: :class:`github.GitAuthor.GitAuthor`
        """
        self._completeIfNotSet(self._author)
        return self._author.value

    @property
    def committer(self):
        """
        :type: :class:`github.GitAuthor.GitAuthor`
        """
        self._completeIfNotSet(self._committer)
        return self._committer.value

    @property
    def html_url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._html_url)
        return self._html_url.value

    @property
    def message(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._message)
        return self._message.value

    @property
    def parents(self):
        """
        :type: list of :class:`github.GitCommit.GitCommit`
        """
        self._completeIfNotSet(self._parents)
        return self._parents.value

    @property
    def sha(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._sha)
        return self._sha.value

    @property
    def tree(self):
        """
        :type: :class:`github.GitTree.GitTree`
        """
        self._completeIfNotSet(self._tree)
        return self._tree.value

    @property
    def url(self):
        """
        :type: string
        """
        self._completeIfNotSet(self._url)
        return self._url.value

    @property
    def _identity(self):
        return self.sha

    def _initAttributes(self):
        self._author = github.GithubObject.NotSet
        self._committer = github.GithubObject.NotSet
        self._html_url = github.GithubObject.NotSet
        self._message = github.GithubObject.NotSet
        self._parents = github.GithubObject.NotSet
        self._sha = github.GithubObject.NotSet
        self._tree = github.GithubObject.NotSet
        self._url = github.GithubObject.NotSet

    def _useAttributes(self, attributes):
        if "author" in attributes:  # pragma no branch
            self._author = self._makeClassAttribute(
                github.GitAuthor.GitAuthor, attributes["author"]
            )
        if "committer" in attributes:  # pragma no branch
            self._committer = self._makeClassAttribute(
                github.GitAuthor.GitAuthor, attributes["committer"]
            )
        if "html_url" in attributes:  # pragma no branch
            self._html_url = self._makeStringAttribute(attributes["html_url"])
        if "message" in attributes:  # pragma no branch
            self._message = self._makeStringAttribute(attributes["message"])
        if "parents" in attributes:  # pragma no branch
            self._parents = self._makeListOfClassesAttribute(
                GitCommit, attributes["parents"]
            )
        if "sha" in attributes:  # pragma no branch
            self._sha = self._makeStringAttribute(attributes["sha"])
        if "tree" in attributes:  # pragma no branch
            self._tree = self._makeClassAttribute(
                github.GitTree.GitTree, attributes["tree"]
            )
        if "url" in attributes:  # pragma no branch
            self._url = self._makeStringAttribute(attributes["url"])



from datetime import datetime
from typing import Any, Dict

from github.GithubObject import NonCompletableGithubObject

class Clones(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def count(self) -> int: ...
    @property
    def timestamp(self) -> datetime: ...
    @property
    def uniques(self) -> int: ...



from typing import Any, Dict

from github.GithubObject import NonCompletableGithubObject
from github.Rate import Rate

class RateLimit(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def core(self) -> Rate: ...
    @property
    def graphql(self) -> Rate: ...
    @property
    def rate(self) -> Rate: ...
    @property
    def search(self) -> Rate: ...



from typing import Dict

from github.GithubObject import NonCompletableGithubObject

class CommitStats(NonCompletableGithubObject):
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, int]) -> None: ...
    @property
    def additions(self) -> int: ...
    @property
    def deletions(self) -> int: ...
    @property
    def total(self) -> int: ...



from datetime import datetime
from typing import Any, Dict, Union

from github.DeploymentStatus import DeploymentStatus
from github.GithubObject import CompletableGithubObject, _NotSetType
from github.NamedUser import NamedUser
from github.PaginatedList import PaginatedList

class Deployment(CompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    def get_statuses(self) -> PaginatedList[DeploymentStatus]: ...
    def get_status(self, id_: int) -> DeploymentStatus: ...
    def create_status(
        self,
        state: str,
        target_url: Union[str, _NotSetType] = ...,
        description: Union[str, _NotSetType] = ...,
    ) -> DeploymentStatus: ...
    @property
    def id(self) -> int: ...
    @property
    def url(self) -> str: ...
    @property
    def sha(self) -> str: ...
    @property
    def task(self) -> str: ...
    @property
    def payload(self) -> Dict[str, Any]: ...
    @property
    def original_environment(self) -> str: ...
    @property
    def environment(self) -> str: ...
    @property
    def description(self) -> str: ...
    @property
    def creator(self) -> NamedUser: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def updated_at(self) -> datetime: ...
    @property
    def statuses_url(self) -> str: ...
    @property
    def repository_url(self) -> str: ...



from typing import Any, Dict, Union

from github.GithubObject import CompletableGithubObject, _NotSetType
from github.NamedUser import NamedUser
from github.Organization import Organization
from github.PaginatedList import PaginatedList
from github.Repository import Repository
from github.TeamDiscussion import TeamDiscussion

class Team(CompletableGithubObject):
    def __repr__(self) -> str: ...
    @property
    def _identity(self) -> int: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    def add_membership(
        self, member: NamedUser, role: Union[str, _NotSetType] = ...
    ) -> None: ...
    def add_to_members(self, member: NamedUser) -> None: ...
    def add_to_repos(self, repo: Repository) -> None: ...
    def delete(self) -> None: ...
    @property
    def description(self) -> str: ...
    def edit(
        self,
        name: str,
        description: Union[str, _NotSetType] = ...,
        permission: Union[str, _NotSetType] = ...,
        privacy: Union[str, _NotSetType] = ...,
    ) -> None: ...
    def get_discussions(self) -> PaginatedList[TeamDiscussion]: ...
    def get_members(
        self, role: Union[str, _NotSetType] = ...
    ) -> PaginatedList[NamedUser]: ...
    def get_repos(self) -> PaginatedList[Repository]: ...
    def has_in_members(self, member: NamedUser) -> bool: ...
    def has_in_repos(self, repo: Repository) -> bool: ...
    @property
    def id(self) -> int: ...
    def invitations(self) -> PaginatedList[NamedUser]: ...
    @property
    def members_count(self) -> int: ...
    @property
    def members_url(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def organization(self) -> Organization: ...
    @property
    def permission(self) -> str: ...
    @property
    def privacy(self) -> str: ...
    def remove_from_members(self, member: NamedUser) -> None: ...
    def remove_from_repos(self, repo: Repository) -> None: ...
    def remove_membership(self, member: NamedUser) -> None: ...
    @property
    def repos_count(self) -> int: ...
    @property
    def repositories_url(self) -> str: ...
    def set_repo_permission(self, repo: Repository, permission: str) -> None: ...
    @property
    def slug(self) -> str: ...
    @property
    def url(self) -> str: ...



from datetime import datetime
from typing import Any, Dict, Optional

from github.GithubObject import NonCompletableGithubObject
from github.NamedUser import NamedUser

class CommitStatus(NonCompletableGithubObject):
    def __repr__(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def context(self) -> str: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def creator(self) -> NamedUser: ...
    @property
    def description(self) -> Optional[str]: ...
    @property
    def id(self) -> int: ...
    @property
    def state(self) -> str: ...
    @property
    def target_url(self) -> Optional[str]: ...
    @property
    def updated_at(self) -> datetime: ...
    @property
    def url(self) -> str: ...



from datetime import datetime
from typing import Any, Dict, Optional, Union

from github.Event import Event
from github.Gist import Gist
from github.GithubObject import CompletableGithubObject, _NotSetType
from github.Membership import Membership
from github.Organization import Organization
from github.PaginatedList import PaginatedList
from github.Permissions import Permissions
from github.Plan import Plan
from github.Project import Project
from github.Repository import Repository
from github.UserKey import UserKey

class NamedUser(CompletableGithubObject):
    def __eq__(self, other: Any) -> bool: ...
    def __hash__(self) -> int: ...
    def __repr__(self) -> str: ...
    @property
    def _identity(self) -> str: ...
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def avatar_url(self) -> str: ...
    @property
    def bio(self) -> Optional[str]: ...
    @property
    def blog(self) -> str: ...
    @property
    def collaborators(self) -> Optional[int]: ...
    @property
    def company(self) -> str: ...
    @property
    def contributions(self) -> int: ...
    @property
    def created_at(self) -> datetime: ...
    @property
    def disk_usage(self) -> Optional[int]: ...
    @property
    def email(self) -> str: ...
    @property
    def events_url(self) -> str: ...
    @property
    def followers(self) -> int: ...
    @property
    def followers_url(self) -> str: ...
    @property
    def following(self) -> int: ...
    @property
    def following_url(self) -> str: ...
    def get_events(self) -> PaginatedList[Event]: ...
    def get_followers(self) -> PaginatedList[NamedUser]: ...
    def get_following(self) -> PaginatedList[NamedUser]: ...
    def get_gists(
        self, since: Union[_NotSetType, datetime] = ...
    ) -> PaginatedList[Gist]: ...
    def get_keys(self) -> PaginatedList[UserKey]: ...
    def get_organization_membership(self, org: int) -> Membership: ...
    def get_orgs(self) -> PaginatedList[Organization]: ...
    def get_projects(self, state: str = ...) -> PaginatedList[Project]: ...
    def get_public_events(self) -> PaginatedList[Event]: ...
    def get_public_received_events(self) -> PaginatedList[Event]: ...
    def get_received_events(self) -> PaginatedList[Event]: ...
    def get_repo(self, name: str) -> Repository: ...
    def get_repos(
        self,
        type: Union[str, _NotSetType] = ...,
        sort: Union[str, _NotSetType] = ...,
        direction: Union[str, _NotSetType] = ...,
    ) -> PaginatedList[Repository]: ...
    def get_starred(self) -> PaginatedList[Repository]: ...
    def get_subscriptions(self) -> PaginatedList[Repository]: ...
    def get_watched(self) -> PaginatedList[Repository]: ...
    @property
    def gists_url(self) -> str: ...
    @property
    def gravatar_id(self) -> str: ...
    def has_in_following(self, following: NamedUser) -> bool: ...
    @property
    def hireable(self) -> bool: ...
    @property
    def html_url(self) -> str: ...
    @property
    def id(self) -> int: ...
    @property
    def invitation_teams_url(self) -> str: ...
    @property
    def inviter(self) -> NamedUser: ...
    @property
    def location(self) -> str: ...
    @property
    def login(self) -> str: ...
    @property
    def name(self) -> str: ...
    @property
    def node_id(self) -> str: ...
    @property
    def organizations_url(self) -> str: ...
    @property
    def owned_private_repos(self) -> Optional[int]: ...
    @property
    def permissions(self) -> Permissions: ...
    @property
    def plan(self) -> Optional[Plan]: ...
    @property
    def private_gists(self) -> Optional[int]: ...
    @property
    def public_gists(self) -> int: ...
    @property
    def public_repos(self) -> int: ...
    @property
    def received_events_url(self) -> str: ...
    @property
    def repos_url(self) -> str: ...
    @property
    def role(self) -> str: ...
    @property
    def site_admin(self) -> bool: ...
    @property
    def starred_url(self) -> str: ...
    @property
    def subscriptions_url(self) -> str: ...
    @property
    def suspended_at(self) -> Optional[datetime]: ...
    @property
    def team_count(self) -> int: ...
    @property
    def total_private_repos(self) -> Optional[int]: ...
    @property
    def twitter_username(self) -> str: ...
    @property
    def type(self) -> str: ...
    @property
    def updated_at(self) -> datetime: ...
    @property
    def url(self) -> str: ...



from datetime import datetime
from typing import Any, Dict, Optional

from github.GithubObject import CompletableGithubObject
from github.Issue import Issue
from github.NotificationSubject import NotificationSubject
from github.PullRequest import PullRequest
from github.Repository import Repository

class Notification(CompletableGithubObject):
    def _initAttributes(self) -> None: ...
    def _useAttributes(self, attributes: Dict[str, Any]) -> None: ...
    @property
    def id(self) -> str: ...
    @property
    def last_read_at(self) -> datetime: ...
    def mark_as_read(self) -> None: ...
    @property
    def reason(self) -> str: ...
    @property
    def repository(self) -> Repository: ...
    @property
    def subject(self) -> NotificationSubject: ...
    @property
    def subscription_url(self) -> str: ...
    @property
    def unread(self) -> Optional[bool]: ...
    @property
    def updated_at(self) -> Optional[datetime]: ...
    @property
    def url(self) -> Optional[str]: ...
    def get_pull_request(self) -> PullRequest: ...
    def get_issue(self) -> Issue: ...



# -*- coding: utf-8 -*-

############################ Copyrights and license ############################
#                                                                              #
# Copyright 2013 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2014 Vincent Jacques <vincent@vincent-jacques.net>                 #
# Copyright 2016 Peter Buckley <dx-pbuckley@users.noreply.github.com>          #
# Copyright 2018 Wan Liuyang <tsfdye@gmail.com>                                #
# Copyright 2018 sfdye <tsfdye@gmail.com>                                      #
#                                                                              #
# This file is part of PyGithub.                                               #
# http://pygithub.readthedocs.io/                                              #
#                                                                              #
# PyGithub is free software: you can redistribute it and/or modify it under    #
# the terms of the GNU Lesser General Public License as published by the Free  #
# Software Foundation, either version 3 of the License, or (at your option)    #
# any later version.                                                           #
#                                                                              #
# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY  #
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS    #
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more #
# details.                                                                     #
#                                                                              #
# You should have received a copy of the GNU Lesser General Public License     #
# along with PyGithub. If not, see <http://www.gnu.org/licenses/>.             #
#                                                                              #
################################################################################

import github.GithubObject
import github.NamedUser


class StatsContributor(github.GithubObject.NonCompletableGithubObject):
    """
    This class represents StatsContributors. The reference can be found here http://developer.github.com/v3/repos/statistics/#get-contributors-list-with-additions-deletions-and-commit-counts
    """

    class Week(github.GithubObject.NonCompletableGithubObject):
        """
        This class represents weekly statistics of a contributor.
        """

        @property
        def w(self):
            """
            :type: datetime.datetime
            """
            return self._w.value

        @property
        def a(self):
            """
            :type: int
            """
            return self._a.value

        @property
        def d(self):
            """
            :type: int
            """
            return self._d.value

        @property
        def c(self):
            """
            :type: int
            """
            return self._c.value

        def _initAttributes(self):
            self._w = github.GithubObject.NotSet
            self._a = github.GithubObject.NotSet
            self._d = github.GithubObject.NotSet
            self._c = github.GithubObject.NotSet

        def _useAttributes(self, attributes):
            if "w" in attributes:  # pragma no branch
                self._w = self._makeTimestampAttribute(attributes["w"])
            if "a" in attributes:  # pragma no branch
                self._a = self._makeIntAttribute(attributes["a"])
            if "d" in attributes:  # pragma no branch
                self._d = self._makeIntAttribute(attributes["d"])
            if "c" in attributes:  # pragma no branch
                self._c = self._makeIntAttribute(attributes["c"])

    @property
    def author(self):
        """
        :type: :class:`github.NamedUser.NamedUser`
        """
        return self._author.value

    @property
    def total(self):
        """
        :type: int
        """
        return self._total.value

    @property
    def weeks(self):
        """
        :type: list of :class:`.Week`
        """
        return self._weeks.value

    def _initAttributes(self):
        self._author = github.GithubObject.NotSet
        self._total = github.GithubObject.NotSet
        self._weeks = github.GithubObject.NotSet

    def _useAttributes(self, attributes):
        if "author" in attributes:  # pragma no branch
            self._author = self._makeClassAttribute(
                github.NamedUser.NamedUser, attributes["author"]
            )
        if "total" in attributes:  # pragma no branch
            self._total = self._makeIntAttribute(attributes["total"])
        if "weeks" in attributes:  # pragma no branch
            self._weeks = self._makeListOfClassesAttribute(
                self.Week, attributes["weeks"]
            )
